/* ---------------------------------------------------------------------------
 * Western Digital (Paradise) chipset revision detection functions 
 * --------------------------------------------------------------------------
 *
 * Copyright (C) 1997 by Brian S. Julin <bri@calyx.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as published 
 * by the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file LGPL.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * ---------------------------------------------------------------------------

 * Version 1.1 Brian S. Julin
 * Major: Copied IBM vga driver from GGI 0.0.9 11/20/97 Snapshot
 *
 * ---------------------------------------------------------------------------
 * 
 * This code was derived from the following sources of information:
 * [1] VGADOC Version 4b Finn Thoegersen
 */

#ifndef _wd_family_c
#define _wd_family_c

#include "WD/wd90c3x.h"
#include "WD/wd90c2x.h"
#include "WD/pvga1a_io.h"
#include "WD/wd_family.h"

int kgi_chipset_wd_detect (struct kgi_wd_chipset *chipset)
{
  int grc_lock, old2, sigidx;
  
  chipset->chiptype = CHIPSET_NOT_WD;

  outbMISC(inbMISC());
  
  grc_lock = rwlockGRC();
  
  if (! iswriteGRC(GR09_PR0A_ADDR,GRC_PR0A)) 
    {
      /*  Unlock again */
      rwunlockGRC();
      
      if (iswriteGRC(GR09_PR0A_ADDR,GRC_PR0A)) 
	{
	  old2 = inbCRT(CRT_UPCRT_LOCK); /*  Unlock WD90Cxx registers */ 
	  outbitsCRT((CR29_WUNLOCK | CR29_RUNLOCK),
		     (CR29_WLOCK_MASK | CR29_RLOCK_MASK),CRT_UPCRT_LOCK);  
	  if (! iswriteCRT(0xFF,CRT_SCRATCH1)) 
	    { 
	      chipset->chiptype = CHIPSET_PVGA1A;
	    }
	  else
	    {
	      outbSEQ(SR06_UNLOCK_UPSEQ,SEQ_UPSEQ_LOCK);
	      if (! iswriteSEQ(SR07_SCRATCH_MASK,SEQ_DISP_CONF)) 
		{ 
		  chipset->chiptype = CHIPSET_WD90C00;
		}
	      else
		{
		  if (! iswriteSEQ(0xFF,SEQ_FIFO_CTL)) 
		    {
		      if (iswriteCRT(0x68,CRT_CHIPSIG1)) 
			{ 
			  chipset->chiptype = CHIPSET_WD90C22;
			}
		      else
			{ 
			  if (iswriteCRT(0x90,CRT_CHIPSIG1)) 
			    { 
			      chipset->chiptype = CHIPSET_WD90C20A;
			    }
			  else
			    { 
			      chipset->chiptype = CHIPSET_WD90C20;
			    }
			}
		      outbCRT(0xA6,CRT_CHIPSIG4);
		      if ((inbCRT(CRT_CHIPSIG2) & 0x20) != 0) 
			{
			  outbCRT(0,CRT_CHIPSIG4);
			}
		    }
		  else
		    {
		      if (iswriteSEQ(SR14_LFB_ADDR_MASK,SEQ_FB_CONTROL)) {
			for (sigidx = CRT_CHIPSIG1; 
			     sigidx <= CRT_CHIPREV6; 
			     sigidx++) {
			  chipset->revstring[sigidx - CRT_CHIPSIG1] = 
			    inbCRT(sigidx);
			};
			chipset->revstring[CRT_CHIPREV6 - CRT_CHIPSIG1 + 1] 
			  = 0; 
			switch ((chipset->revstring[5] << 8) + 
				chipset->revstring[6])
			  {
			  case 0x3234: 
			    chipset->chiptype = CHIPSET_WD90C24;
			    break;
			  case 0x3236: 
			    chipset->chiptype = CHIPSET_WD90C26A;
			    break;
			  case 0x3330: 
			    chipset->chiptype = CHIPSET_WD90C30;
			    break;
			  case 0x3331: 
			    chipset->chiptype = CHIPSET_WD90C31;
			    break;
			  case 0x3333: 
			    chipset->chiptype = CHIPSET_WD90C33;
			    break;
			  default: 
			    if (inbCRT(CRT_CHIPSIG1) == 'W'
				&& inbCRT(CRT_CHIPSIG2) == 'D') {
			      chipset->chiptype = CHIPSET_UNKNOWN_WD;
			    }
			    else 
			      { 
				chipset->chiptype = CHIPSET_NOT_WD;
			      };
			  }
		      }
		      else
			{
			  if (! iswriteSEQ(4,SEQ_FIFO_CTL)) 
			    { 
			      chipset->chiptype = CHIPSET_WD90C10;
			    }
			  else
			    {
			      chipset->chiptype = CHIPSET_WD90C11;
			    }
			}
		    }
		}
	    }
	  outbCRT(old2,CRT_UPCRT_LOCK);
	}
    };
  /* Restore lock to original state */
  outbGRC(grc_lock,GRC_UPGRC_LOCK);
  
  return(!(chipset->chiptype == CHIPSET_UNKNOWN_WD));
  
}

void kgi_chipset_wd_findram (struct kgi_wd_chipset *chipset) {
  uint8 grc_lock;
  
  grc_lock = runlockGRC();
  switch ((inbGRC(GRC_MEMORY)) & GR0B_RAMSIZE) 
    {
    case GR0B_256K:
      chipset->chipset.ramsize = 256 * 1024;
      chipset->features &= ~BANKING_RAM;
      break;
    case GR0B_256K_BANKED:
      chipset->chipset.ramsize = 256 * 1024;
      chipset->features |= BANKING_RAM;
      break;
    case GR0B_512K_BANKED:
      chipset->chipset.ramsize = 512 * 1024;
      chipset->features |= BANKING_RAM;
      break;
    case GR0B_1M_BANKED:
      chipset->chipset.ramsize = 1024 * 1024;
      chipset->features |= BANKING_RAM;
      break;
    };
  chipset->chipset.ramsize = ((chipset->chiptype == CHIPSET_WD90C33) 
			      && (inbCRT(CRT_OVERFLOW3) & CR3E_2MB_RAM)) ?
    2048 * 1024 : chipset->chipset.ramsize;

  /* Restore lock to original state */
  outbGRC(grc_lock, GRC_UPGRC_LOCK);
}

#endif /* not _wd_family_c */
