/* ---------------------------------------------------------------------------
 * PVGA1 I/O - (inline functions, extern variable and function headers)
 * --------------------------------------------------------------------------
 *
 * Copyright (C) 1997 by Brian S. Julin, <bri@calyx.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as published 
 * by the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * You should have received a copy of the GNU Library General Public 
 * License along with this program; see the file LGPL.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * ---------------------------------------------------------------------------
 *
 * Revision 0.1 
 * Major: Copied from IBM vga_io.inc in GGI 0.0.9
 *
 * ---------------------------------------------------------------------------
 *	This code was derived from the following source of information:
 *	[1]	XFree86 vga driver
 */

#ifndef _pvga1a_io_h
#define _pvga1a_io_h

#include <asm/io.h>
#include <WD/temporary_hacks.h>
/* #include <ggi/kgi.h> */
#include <WD/pvga1a.h>

/* Decide whether we are compiling or including this library */
#undef TERNAL
#ifdef _pvga1a_io_c
#define TERNAL
#else
#define TERNAL extern
TERNAL struct io_region wd_wakeup;
TERNAL struct mem_region wd_mem_graph;
TERNAL struct mem_region wd_mem_text;
TERNAL void outPR0A(uint16);
TERNAL void outPR0B(uint16);
#endif /* not _pvga1a_io_c */

#define TRUE 1
#define FALSE 0

/*-----------------------------------------------------------------------*/
/*             Set up data structure for holding chip state              */
/*-----------------------------------------------------------------------*/
#define NoGRCRegs       0x10          /* all paradise have this many */
#define NoATCRegs       0x15          /* all paradise have this many */
#ifndef _wd90c00_io_h
#define NoCRTRegs       0x25          /* wd90c00+ chipsets have more */
#endif /* not _wd90c00_io_h */
#ifndef _wd90c1x_io_h
#define NoSEQRegs       0x05          /* wd90c1x+ chipsets have more */
#endif /* not _wd90c1x_io_h */

struct statetable
{
  uint8 MISC;
  uint8 SEQ[NoSEQRegs];
  uint8 CRT[NoCRTRegs];
  uint8 GRC[NoGRCRegs];
  uint8 ATC[NoATCRegs];
};

/* These can have variable locations depending MONO/COLOR */
extern uint16 INS1r, CRTI, CRTD;


/*-----------------------------------------------------------------------*/
/*                   Sleep/Lock Manipulation Functions                   */
/*-----------------------------------------------------------------------*/
static inline void pvga_wakeup (void) {
  io_out8((io_in8(WD_WAKEUP) | WD_WAKEY_WAKEY), WD_WAKEUP);
}

static inline void pvga_sleep (void) {
  /* Need to figure out bus type to implement this ?  are there VLB PVG1As?*/
}


/*-----------------------------------------------------------------------*/
/*                 Indexed Register Functions                            */
/*-----------------------------------------------------------------------*/

static inline uint8 inb_idx_reg(uint16 idx, uint16 dat, uint8 reg) 
{ outb(reg, idx); return inb(dat); }

static inline void outb_idx_reg(uint8 val, uint16 idx, uint8 reg) 
{ outw((val << 8) | reg, idx); }

static inline uint8 outbits_idx_reg(uint8 val, uint8 bitmask,
				   uint16 idx, uint16 dat, uint8 reg) 
{ 
  int oldval;
  oldval = inb_idx_reg(idx,dat,reg);
  outb_idx_reg((oldval & ~bitmask) | (val & bitmask), idx, reg); 
  return(oldval);
}

TERNAL int iswrite_idx_reg(uint8, uint16, uint16, uint8);




static inline uint8 inbGRC(uint8 reg)
{ return inb_idx_reg(GRCI,GRCD,reg); }

static inline void outbGRC(uint8 value, uint8 reg)
{ outb_idx_reg(value, GRCI, reg); }

static inline uint8 outbitsGRC(uint8 val, uint8 bitmask, uint8 reg) 
{ return (outbits_idx_reg(val, bitmask, GRCI, GRCD, reg)); }

static inline int iswriteGRC(uint8 bitmask, uint8 reg) 
{ return iswrite_idx_reg(bitmask, GRCI, GRCD, reg); }

static inline uint8 rlockGRC(void) 
{
  /* This doesn't work.  When I witness a card that actually does
     work, I will implement code to guess status of RO read lock bit */
  return(outbitsGRC(GR0F_RLOCK, GR0F_RLOCK, GRC_UPGRC_LOCK));
}

static inline int runlockGRC(void) 
{ return(outbitsGRC(0, GR0F_RLOCK, GRC_UPGRC_LOCK)); }

static inline int wlockGRC(void) 
{ return(outbitsGRC(GR0F_WLOCK, GR0F_WLOCK_MASK, GRC_UPGRC_LOCK)); }

static inline int wunlockGRC(void) 
{ return(outbitsGRC(GR0F_WUNLOCK, GR0F_WLOCK_MASK, GRC_UPGRC_LOCK)); }

static inline int rwlockGRC(void) 
{ 
  return(outbitsGRC((GR0F_WLOCK | GR0F_RLOCK),
		    (GR0F_WLOCK_MASK | GR0F_RLOCK), 
		    GRC_UPGRC_LOCK));
}

static inline int rwunlockGRC(void) 
{
  return(outbitsGRC((GR0F_WUNLOCK),
		    (GR0F_WLOCK_MASK | GR0F_RLOCK), 
		    GRC_UPGRC_LOCK));
}


TERNAL void save_GRC(uint8 *);
TERNAL void restore_GRC(uint8 *);



static inline uint8 inbCRT(uint8 reg)
{ return inb_idx_reg(CRTI,CRTD,reg); }

static inline void outbCRT(uint8 value, uint8 reg)
{ outb_idx_reg(value, CRTI, reg); }

static inline uint8 outbitsCRT(uint8 val, uint8 bitmask, uint8 reg) 
{ return (outbits_idx_reg(val, bitmask, CRTI, CRTD, reg)); }

static inline int iswriteCRT(uint8 bitmask, uint8 reg) 
{ return iswrite_idx_reg(bitmask, CRTI, CRTD, reg); }

#ifndef _wd90c00_io_h  /* wd90c00 has to unlock upper CRT registers, too */
static inline int wunlockCRT(void) 
{ 
  int lockval1, lockval2;
  lockval1 = rwunlockGRC();  /* Some CRT locks kept in upper GRC,
                               which can in turn be locked -- pain */
  lockval2 = inbGRC(GRC_CRTC_CTL);
  outbGRC((lockval2 | GR0D_CR11_NOLOCK /* Disables traditional VGA lock */)
	  & ~(GR0D_VTIMING_LOCK | GR0D_HTIMING_LOCK), GRC_CRTC_CTL);
  outbGRC(lockval1, GRC_UPGRC_LOCK); /* Restore locking of upper GRC */
  return(lockval2);
}

static inline int wlockCRT(void) 
{ 
  int lockval1, lockval2;
  lockval1 = rwunlockGRC();  /* Some CRT locks kept in upper GRC,
                               which can in turn be locked -- pain */
  lockval2 = inbGRC(GRC_CRTC_CTL);
  outbGRC((lockval2 | GR0D_CR11_NOLOCK /* Disables traditional VGA lock */
	  | GR0D_VTIMING_LOCK | GR0D_HTIMING_LOCK), GRC_CRTC_CTL);
  outbGRC(lockval1, GRC_UPGRC_LOCK); /* Restore locking of upper GRC */
  return(lockval2);
}

#endif /* notdef _wd90c00_io_h */


TERNAL void save_CRT(uint8 *);
TERNAL void restore_CRT(uint8 *);



static inline uint8 inbSEQ(uint8 reg)
{ return inb_idx_reg(SEQI,SEQD,reg); }

static inline void outbSEQ(uint8 value, uint8 reg)
{ outb_idx_reg(value, SEQI, reg); }

static inline uint8 outbitsSEQ(uint8 val, uint8 bitmask, uint8 reg) 
{ return (outbits_idx_reg(val, bitmask, SEQI, SEQD, reg)); }

static inline int iswriteSEQ(uint8 bitmask, uint8 reg) 
{ return iswrite_idx_reg(bitmask, SEQI, SEQD, reg); }

TERNAL void save_SEQ(uint8 *);
TERNAL void restore_SEQ(uint8 *);


static inline uint8 inbATC(uint8 reg)
{ return inb_idx_reg(ATCI,ATCDr,reg); }

static inline void outbATC(uint8 value, uint8 reg)
{ outb(reg, ATCI); outb(value, ATCDw); }

static inline void outbitsATC(uint8 val, uint8 bitmask, uint8 reg) 
{ outbATC((inbATC(reg) & (~ bitmask)) + (val & bitmask), reg);}

TERNAL int iswriteATC(uint8, uint8);
TERNAL void save_ATC(uint8 *);
TERNAL void restore_ATC(uint8 *);

TERNAL void save_state(struct statetable *);
TERNAL void restore_state(struct statetable *);



/*-----------------------------------------------------------------------*/
/*                 Other External Register Functions                     */
/*-----------------------------------------------------------------------*/

static inline void outbMISC(uint8 value)
{
  outb(value, MISCw);			/* Put value      */
  if (value & MISC_COLOR_IO)		/* Adjust IO base (blech) */
    {      
      CRTI = CRTIc; 
      CRTD = CRTDc; 
      INS1r = INS1rc; 
    }
  else
    {
      CRTI = CRTIm;
      CRTD = CRTDm;
      INS1r = INS1rm;
    };
}

static inline uint8 inbMISC (void) {
  return io_in8(MISCr);
}

static inline int wunlockMISC(void) 
{ 
  int lockval1, lockval2;
  lockval1 = rwunlockGRC();  /* Some CRT locks kept in upper GRC,
                               which can in turn be locked -- pain */
  lockval2 = inbGRC(GRC_CRTC_CTL);
  outbGRC(lockval2 & ~(GR0D_VPOL_LOCK | GR0D_HPOL_LOCK), GRC_CRTC_CTL);
  outbGRC(lockval1, GRC_UPGRC_LOCK); /* Restore locking of upper GRC */
  return(lockval2);
}

static inline int wlockMISC(void)   /* Note: Only locks HPOL/VPOL */
{ 
  int lockval1, lockval2;
  lockval1 = rwunlockGRC();  /* Some CRT locks kept in upper GRC,
                               which can in turn be locked -- pain */
  lockval2 = inbGRC(GRC_CRTC_CTL);
  outbGRC((lockval2 | GR0D_VPOL_LOCK | GR0D_HPOL_LOCK), GRC_CRTC_CTL);
  outbGRC(lockval1, GRC_UPGRC_LOCK); /* Restore locking of upper GRC */
  return(lockval2);
}


/* Module option for use when running wd90c1x+ with the pvga1a driver */
#ifndef _pvga1a_c
extern int invclk;
#endif _pvga1a_c

#endif /* _pvga1a_io_h */
