/* ---------------------------------------------------------------------------
**	Chips & Technologies 655xx ramdac driver
** ---------------------------------------------------------------------------
**
**	Copyright (C) 1997	Stephan Meyer
**
**	This program is free software; you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation; either version 2, or (at your option)
**	any later version.
**
**	You should have received a copy of the GNU General Public License
**	along with this program; see the file COPYING.  If not, write to
**	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
** ---------------------------------------------------------------------------
**
**	$Log: 655xx.c,v $
**	Revision 1.4  1997/12/18 23:51:24  becka
**	More ramdacs set right ...
**	
**	Revision 1.3  1997/12/14 21:56:17  becka
**	uint/sint namespace fixed ... please test ...
**	
**	Revision 1.2  1997/12/03 15:47:09  seeger
**	small bug in header includes fixed.
**	
**	Revision 1.1  1997/10/22 17:35:46  seeger
**	Initial C&T driver checkin
**	
*/

#include <ggi/maintainers.h>
#define	MAINTAINER	Stephan_Meyer
#define	DRIVER_NAME	"Chips 655xx ramdac driver"
#define	DRIVER_REV	"$Revision: 1.4 $"
#define	DEBUG_LEVEL	0

#include <ggi/module.h>

#include "Chips/chips.h"

#define DAC_PRECISION   6

struct kgi_ramdac ramdac = 
{
	"Chips & Technologies",		/* Manufacturer		*/
	"655xx",			/* Model		*/
	GM_TEXT16 | GM_8BIT |
	GM_16BIT | GM_24BIT,		/* Supported modes	*/
	0,			 	/* Flags		*/
	{ 0xFFFF, 0xFFFF },		/* Maximum resolution	*/
	DAC_8BIT_PB,  			/* Device type		*/
	{       0,  32000000 },		/* Normal mode		*/
	{       0,         0 },		/* 2xclk mode		*/
	{       /* autodetect */
		{	  0,  25175000 },	/* GT_TEXT16	*/
		{	  0,         0 },	/* GT_TEXT32	*/
		{         0,         0 },	/* GT_1BIT	*/
		{	  0,         0 },	/* GT_4BIT	*/
		{         0,         0 },	/* GT_8BIT	*/	
		{	  0,         0 },	/* GT_15BIT	*/
		{ 	  0,         0 },	/* GT_16BIT	*/
		{         0,         0 },	/* GT_24BIT	*/
		{	  0,         0 }	/* GT_32BIT	*/
	}
};

extern int	chips_model;
extern ChipsRegs chips_resetregs;

/* taken from the cirrus ramdac driver */

static void dac_set_clut(struct kgi_display *dpy, ggi_clut *clut,
			 ggi_uint first, ggi_uint cnt) {
	ushort *data=(ushort *)(clut->data+first);
	cnt*=3;
	outb(first,0x3C8);
	while (cnt--) outb(*(data++)>>10,0x3C9);
}

static void dac_get_clut(struct kgi_display *dpy, ggi_clut *clut,
			 ggi_uint first, ggi_uint cnt) {
	ushort *data=(ushort *)(clut->data+first);
	cnt*=3;
	outb(first,0x3C7);
	while (cnt--) *(data++)=inb(0x3C9)*1040;
}

/* GGI interface */

int kgim_ramdac_init(struct kgi_card *card, struct kgi_display *dpy)
{
	NOTICE(DRIVER_NAME DRIVER_REV);
	switch (chips_model) {
		case CHIPS_65545 :
			if (chips_resetregs.extension[0x6C] & 2) {
				ramdac.clock[GT_8BIT].max=68000000;
				ramdac.clock[GT_16BIT].max=34000000;
				ramdac.clock[GT_24BIT].max=22667000;
			} else {
				ramdac.clock[GT_8BIT].max=56000000;
				ramdac.clock[GT_16BIT].max=28322000;
				ramdac.clock[GT_24BIT].max=18667000;
			}
			break;
		case CHIPS_65548 :
			ramdac.clock[GT_8BIT].max=80000000;
			ramdac.clock[GT_16BIT].max=40000000;
			ramdac.clock[GT_24BIT].max=26667000;
			break;
		case CHIPS_65550 :
		case CHIPS_65554 :
			io_out8(0x0A,0x3D0);
			if (io_in8(0x3D1) & 2) {
				ramdac.clock[GT_8BIT].max=110000000;
				ramdac.clock[GT_16BIT].max=110000000;
				ramdac.clock[GT_24BIT].max=110000000;
			} else {
				ramdac.clock[GT_8BIT].max=80000000;
				ramdac.clock[GT_16BIT].max=80000000;
				ramdac.clock[GT_24BIT].max=80000000;
			}
			break;
	}

	card->ramdac=&ramdac;

	return EOK;
}

void kgim_ramdac_done(struct kgi_display *dpy)
{
	NOTICE(DRIVER_NAME " removed.");
}

int kgim_ramdac_check_mode(struct kgi_display *dpy, struct kgi_mode *mode,
	enum kgi_tm_cmd cmd)
{
	ggi_uint newclock = mode->dclk;
  
	switch (mode->request.graphtype) {

	case GT_TEXT16:
		break;

	case GT_8BIT:
		break;

	case GT_16BIT:
		break;

	case GT_24BIT:
		break;
  
	default:
  		DEBUG2("couldn't handle graphtype %i", 
		       mode->request.graphtype);
		return -E(RAMDAC,MODE_CHECK);
	}

	switch (cmd) {

	case CMD_PROPOSE:
		mode->lclk.mul = mode->clk.mul = 1;
		mode->lclk.div = mode->clk.div = 1;
		return EOK;

	case CMD_LOWER:

		if ((newclock < ramdac.clock[mode->request.graphtype].min) ||
			(newclock < ramdac.normal.min)) {

			DEBUG2("DCLK too low (%i Hz).", newclock);
			return -E(RAMDAC,CLK_LOW_LIMIT);
		}

		if (ramdac.clock[mode->request.graphtype].max < newclock) {

			newclock = ramdac.clock[mode->request.graphtype].max;
		}
		mode->dclk = newclock;
		return EOK;

	case CMD_RAISE:
		if ((newclock > ramdac.clock[mode->request.graphtype].max) ||
			(newclock > ramdac.normal.max)) {

			DEBUG2("DCLK too high (%i Hz).", newclock);
			return -E(RAMDAC,CLK_HIGH_LIMIT);
		}

		if (newclock < ramdac.clock[mode->request.graphtype].min) {

			newclock = ramdac.clock[mode->request.graphtype].min;
		}

		if (newclock < ramdac.normal.min) {

			newclock = ramdac.normal.min;
		}

		mode->dclk = newclock;
		return EOK;

     case CMD_CHECK:
		if ((mode->lclk.mul != 1) || (mode->lclk.div != 1) ||
		    (mode->clk.mul != 1) || (mode->clk.div != 1)) {

			return -E(RAMDAC,MODE_CHECK);
		}

		if ((mode->dclk < ramdac.clock[mode->request.graphtype].min) ||
		    (mode->dclk < ramdac.normal.min)) {

			return -E(RAMDAC,CLK_LOW_LIMIT);
		}

		if ((mode->dclk > ramdac.clock[mode->request.graphtype].max) ||
		    (mode->dclk > ramdac.normal.max)) {
			
			return -E(RAMDAC,CLK_HIGH_LIMIT);
		}

		return EOK;

	default:
		INTERNAL_ERROR;
		return -E(RAMDAC,BOGUS);
	}
}

void kgim_ramdac_set_mode(struct kgi_display *dpy, struct kgi_mode *mode)
{
	switch (mode->request.graphtype) {

	case GT_TEXT16:
	case GT_8BIT:
	case GT_16BIT:
	case GT_24BIT:
		break;

	default:
		INTERNAL_ERROR;
	}
	dpy->set_clut = dac_set_clut;
}

int kgim_ramdac_command(struct kgi_graphic *graph, ggi_uint cmd, ggi_uint arg)
{
	/* This may set 
	 * -  Hardware cursor
	 * -  CLUT (or gamma correction table)
	 */
	ggi_clut clut;

	switch (cmd) { 

	case RAMDAC_SETCLUT:
		clut.size = 256;
		clut.data = (ggi_color *)(graph->io_buf);
		dac_set_clut(graph->dev.dpy, &clut, 0, 256);
		return 0;

	case RAMDAC_GETCLUT:
		clut.size = 256;
		clut.data = (ggi_color *)(graph->io_buf);
		graph->io_res = graph->io_buf;
		dac_get_clut(graph->dev.dpy, &clut, 0, 256);
		return 0;
	}
  	return -E(RAMDAC, NOSUP_ALWAYS_CANT);  /* The ramdac can't do this */
}

