/*\
|*| ----------------------------------------------------------------------------
|*|	KGI console handling
|*| ----------------------------------------------------------------------------
|*|
|*|	Copyright (C) 1996 by Steffen Seeger, [seeger@physik.tu-chemnitz.de]
|*|
|*|	This program is free software; you can redistribute it and/or modify
|*|	it under the terms of the GNU General Public License as published by
|*|	the Free Software Foundation; either version 2, or (at your option)
|*|	any later version.
|*|
|*|	You should have received a copy of the GNU General Public License
|*|	along with this program; see the file COPYING.  If not, write to
|*|	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
|*|
|*| ----------------------------------------------------------------------------
|*|	MAINTAINER	"Steffen Seeger,[seeger@physik.tu-chemnitz.de]"
\*/

#ifndef _GGI_CONSOLE_H
#define _GGI_CONSOLE_H

#define	MAX_NPAR	16

enum modes
{
	smooth_scroll,	/* smooth(slow) scroll		*/
	reverse_video,	/* black on white		*/
	origin,		/* cursor origin mode		*/
	auto_wrap,	/* automatic line wrap		*/

	auto_repeat,	/* repeat chars automatically	*/
	reverse_wrap,	/* allow reverse wrapping	*/
	alt_screen,	/* use alternate screen		*/
	show_cursor,	/* show/hide cursor		*/

	scroll_last_mode /* NOTE: there *must* not be more than 32 modes! */
};

enum flags
{
	need_wrap,	/* cursor at rightmost column	*/
	scrolled,	/* ORIGIN or OFFSET modified	*/
	no_hardscroll,	/* small hardware framebuffer	*/
	splitline,	/* use wrapping mode, don't copy*/
	compose,	/* key compose in progress	*/

	scroll_last_flag /* NOTE: there *must* not be more than 32 flags! */
};


enum 
{	COL_BLACK,	COL_BLUE,	COL_GREEN,	COL_CYAN,
	COL_RED,	COL_MAGENTA,	COL_BROWN,	COL_LIGHTGRAY,
	COL_DARKGRAY,	COL_LIGHTBLUE,	COL_LIGHTGREEN,	COL_LIGHTCYAN,
	COL_LIGHTRED,	COL_LIGHTMAGENTA,COL_YELLOW,	COL_WHITE
};

#ifdef __KERNEL__
extern inline void scroll_paranoia_check(void)
{
	if (scroll_last_flag >= 32) {

		panic(__FILE__": scroll_last_flag >= 32");
	}

	if (scroll_last_mode >= 32) {

		panic(__FILE__": scroll_last_mode >= 32");
	}
}
#endif

#if 0 /*************************************************************/


struct con_curr_state
{	ggi_uint		x,y;		/* write position		*/
	ggi_attr	attrfl;		/* attributes in effect		*/
	uchar		colors[8];	/* color attributes		*/
	ggi_uint		charset;	/* current charset		*/
	uchar		g[4];		/* charset for G0...G3		*/
};

#endif /*****************************************************************/


struct kgi_console;

struct scroller_state
{
	int	(*do_write)(struct kgi_console *, int, const char *, int);
					/* terminal emulation to use	*/

	ushort 	*bgbuf;			/* bg pixel buffer		*/
	ggi_uint	bglines;		/* number of lines in bg_buffer */

	ggi_uint	from, to;		/* area modified		*/
	int	offset;			/* scroll back info		*/
	int	last_offset;		/* offset we have synced to	*/

	ggi_uint	sizex;			/* logical size			*/
	ggi_uint	sizey;
	ggi_uint	fb_size;		/* size of the framebuffer	*/
	ggi_uint	fsize;			/* size of the visible area	*/

	ggi_uint	top, bottom;		/* scrolling region		*/
	ggi_uint	origin;			/* origin line			*/
	int	x,y;			/* writing position		*/
	ggi_uint	org;			/* origin pixel			*/
	ggi_uint	wpos;			/* pixel to write		*/
	ggi_uint	pos;			/* wpos % fb_size		*/

	ggi_attr	attrfl;		/* current attribute flags	*/
	ggi_uint		fg, bg;		/* fg and bg color		*/
	ggi_pixel	attr;		/* current attributes		*/
	ggi_pixel	erase;		/* pixelval for blank char	*/

	ggi_uint		bell_pitch;	/* frequency in Hz, please	*/
	ggi_uint		bell_duration;	/* duration in ms, please	*/

	ggi_uint		dead;		/* dead key symbol		*/

	ulong		flags;		/* flags for the scroller	*/
	ulong		mode;		/* mode status for the scroller	*/
};

#include <ggi/term/xterm.h>

struct kgi_console
{
	struct kgi_device	dev;	/* virtual device state		*/
	struct scroller_state	scroll;	/* scroller state		*/

	union {

		struct xterm_state	xterm;

	}			term;	/* terminal emulator state	*/
};


#ifdef __KERNEL__

extern int printk_console;
extern ulong con_need_sync[(MAX_NR_CONSOLES+sizeof(ulong)*8-1)/(sizeof(ulong)*8)];
extern void console_print(const char *s);
extern void dev_console_init(void);

#ifdef NEED_CONSOLE_DEFS

#define	DEV		(cons->dev)
#define	TTY		(DEV.tty)
#define	FB		(DEV.fb)
#define	FB_SIZE_X	(DEV.fb->size.x)
#define	FB_SIZE_Y	(DEV.fb->size.y)
#define	FONT		(DEV.font)
#define	DPP_X		(DEV.mode.dpp.x)
#define	DPP_Y		(DEV.mode.dpp.y)
#define	VIRTUAL_X	(DEV.mode.virt.x)
#define	VIRTUAL_Y	(DEV.mode.virt.y)
#define	VISIBLE_X	(DEV.mode.visible.x)
#define	VISIBLE_Y	(DEV.mode.visible.y)
#define	CURSOR_X	(DEV.cursor.x)
#define	CURSOR_Y	(DEV.cursor.y)
#define	POINTER_X	(DEV.pointer.x)
#define	POINTER_Y	(DEV.pointer.y)


#define	SCROLL		(cons->scroll)

#define	FLAG(x)		(SCROLL.flags & (1 << x))
#define	SET_FLAG(x)	SCROLL.flags |= (1 << x)
#define	CLEAR_FLAG(x)	SCROLL.flags &= ~(1 << x)
#define	TOGGLE_FLAG(x)	SCROLL.flags ^= (1 << x)

#define	MODE(x)		(SCROLL.mode & (1 << x))
#define	SET_MODE(x)	SCROLL.mode |= (1 << x)
#define	CLEAR_MODE(x)	SCROLL.mode &= ~(1 << x)
#define	TOGGLE_MODE(x)	SCROLL.mode ^= (1 << x)

#define	BGBUF		(SCROLL.bgbuf)
#define	BGLINES		(SCROLL.bglines)

#define	DO_WRITE	(SCROLL.do_write)
#define	FROM		(SCROLL.from)
#define	TO		(SCROLL.to)
#define	OFFSET		(SCROLL.offset)
#define	LAST_OFFSET	(SCROLL.last_offset)

#define	SIZE_X		(SCROLL.sizex)
#define	SIZE_Y		(SCROLL.sizey)
#define	FB_SIZE		(SCROLL.fb_size)
#define	FSIZE		(SCROLL.fsize)

#define	TOP		(SCROLL.top)
#define	BOTTOM		(SCROLL.bottom)
#define	ORIGIN		(SCROLL.origin)
#define	X		(SCROLL.x)
#define	Y		(SCROLL.y)
#define	ORG		(SCROLL.org)
#define	WPOS		(SCROLL.wpos)
#define	POS		(SCROLL.pos)

#define	ATTRFL		(SCROLL.attrfl)

	#define	INTENSITY	(ATTRFL & ATTR_INTENSITY)
	#define	UNDERLINE	(ATTRFL & ATTR_UNDERLINE)
	#define	BOLD		(ATTRFL & ATTR_BOLD)
	#define	ITALIC		(ATTRFL & ATTR_ITALIC)
	#define	REVERSE		(ATTRFL & ATTR_REVERSE)
	#define	BLINK		(ATTRFL & ATTR_BLINK)

#define	ATTR		(SCROLL.attr)
#define	ERASE		(SCROLL.erase)

#define	BELL_PITCH	(SCROLL.bell_pitch)
#define	BELL_DURATION	(SCROLL.bell_duration)


/*	the scroller interface
*/

extern void scroll_up(struct kgi_console *cons, ggi_uint t, ggi_uint b, ggi_uint n);
extern void scroll_down(struct kgi_console *cons, ggi_uint t, ggi_uint b, ggi_uint n);
extern void scroll_erase_display(struct kgi_console *cons, int arg);
extern void scroll_erase_line(struct kgi_console *cons, int arg);
extern void scroll_insert_chars(struct kgi_console *cons, ggi_uint n);
extern void scroll_delete_chars(struct kgi_console *cons, ggi_uint n);
extern void scroll_erase_chars(struct kgi_console *cons, ggi_uint n);
extern void scroll_backward(struct kgi_console *cons, int lines);
extern void scroll_forward(struct kgi_console *cons, int lines);
extern void scroll_lf(struct kgi_console *cons);
extern void scroll_reverse_lf(struct kgi_console *cons);
extern void scroll_sync(struct kgi_console *cons);
extern void scroll_gotoxy(struct kgi_console *cons, int new_x, int new_y);


/*	There is a bug in GCC that disables checking for warnings in
**	'extern inline' functions. So for we force inline functions to be
**	declared 'extern' and not to be inlined to trap silly errors not
**	found otherwise when DEBUG is declared!
*/

#ifdef DEBUG

#define	INLINE 

extern void scroll_modified(struct kgi_console *cons,ggi_uint from,ggi_uint to);
extern void scroll_update_attr(struct kgi_console *cons);
extern void scroll_update_gadgets(struct kgi_console *cons);
extern void scroll_cr(struct kgi_console *cons);
extern void scroll_bs(struct kgi_console *cons);
extern void scroll_write(struct kgi_console *cons, unicode c);
extern void scroll_need_sync(struct kgi_console *cons);
extern void scroll_mksound(struct kgi_console *cons, int pitch, int duration);

#else

#define	INLINE	extern inline

#endif

#if (!defined(DEBUG) || defined(INCLUDE_FROM_CONSOLE_C))

INLINE void scroll_modified(struct kgi_console *cons, ggi_uint from, ggi_uint to)
{
	ASSERT(from <= to);

	if (from < FROM) {

		FROM = from;
	}

	if (TO < to) {

		TO = to;
	}
}

INLINE void scroll_update_attr(struct kgi_console *cons)
{
	ATTR  = fb_attr_to_pixel(FB, ATTRFL);
	ERASE = fb_fpos_to_pixel(FB, kgi_fontpos(&FONT,
					TTY ? ERASE_CHAR(TTY) : 0x20)) | ATTR;
}

INLINE void scroll_update_gadgets(struct kgi_console *cons)
{
	CURSOR_X = X * DPP_X;
	CURSOR_Y = Y * DPP_Y;

	DEV.state &= ~(KGI_POINTER_TO_SHOW | KGI_CURSOR_TO_SHOW);

	if (! OFFSET) {

		if (MODE(show_cursor)) {

			DEV.state |= KGI_CURSOR_TO_SHOW;
		}

		if (MODE(x10_mouse) | MODE(report_mouse) | MODE(track_mouse)) {

			DEV.state |= KGI_POINTER_TO_SHOW;
		}	
	}
}

INLINE void scroll_cr(struct kgi_console *cons)
{
	WPOS -= X;
	POS -= X;
	X = 0;
	CLEAR_FLAG(need_wrap);
}

INLINE void scroll_bs(struct kgi_console *cons)
{
	if (X) {
		X--;
		WPOS--;
		POS--;
		CLEAR_FLAG(need_wrap);
	}
}

INLINE void scroll_write(struct kgi_console *cons, unicode c)
{
	fb_update(FB, POS, ATTR | kgi_fontpos(&(FONT), c));

	if (X == (SIZE_X - 1)) {

		SET_FLAG(need_wrap);
		return;
	}

	X++;
	WPOS++;
	POS++;
}

INLINE void scroll_need_sync(struct kgi_console *cons)
{
	set_bit(DEV.id, con_need_sync);
	mark_bh(CONSOLE_BH);
}

INLINE void scroll_mksound(struct kgi_console *cons, int pitch, int duration)
{
	DEBUG0("scroll_mksound not implemented yet.");
}

#endif	/* #if (!defined(DEBUG) || defined(INCLUDE_FROM_CONSOLE_C)) */

#endif	/* #ifdef NEED_CONSOLE_DEFS */

#endif	/* #ifdef __KERNEL__ */

#endif	/* #ifdef _GGI_CONSOLE_H */
