/*\
|*| --------------------------------------------------------------------------
|*|     GGI command definitions
|*| --------------------------------------------------------------------------
|*|
|*|	Copyright (C) 1997 by Andreas Beck, [becka@hp.rz.uni-duesseldorf.de]
|*|			  and Steffen Seeger, [seeger@physik.tu-chemnitz.de]
|*|
|*|	This program is free software; you can redistribute it and/or modify
|*|	it under the terms of the GNU General Public License as published by
|*|	the Free Software Foundation; either version 2, or (at your option)
|*|	any later version.
|*|
|*|	You should have received a copy of the GNU General Public License
|*|	along with this program; see the file COPYING.  If not, write to
|*|	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
|*|
|*| ---------------------------------------------------------------------------
\*/

/* MAINTAINER	ggi-develop@eskimo.com */

#ifndef _ggi_commands_h
#define _ggi_commands_h

#ifdef __KERNEL__
#include <linux/fs.h>
#include <ggi/kgi.h>
#include <linux/types.h>
#include <linux/ioctl.h>
#else
#include <sys/types.h>	
#include <sys/ioctl.h>
#endif

#include <ggi/ggi.h>

/*\
|*|	Definitions for device-registration
\*/

#define	DEVICEFILE_MAJOR	60
#define DEVICEFILE_NAME		"graphics"
#define	DEVICEFILE_MAGIC	((GGI_VERSION << 8) | 0xAB)

/*\
|*|	The ioctl's
|*|
|*|	Bit coding is according to <asm/ioctl.h> now. The <type>
|*|	gives the subsystem and the <nr> the command.
|*|
|*| !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
|*| !!!	NOTE: DON NOT CHANGE WITHOUT CHANGING THE KERNEL DRIVER INTERFACE !!!
|*| !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
\*/ 

#define IOC_DRIVER		0x00	/* kernel driver	*/
#define IOC_MONITOR		0x01	/* monitor driver	*/
#define IOC_RAMDAC		0x02	/* ramdac driver	*/
#define IOC_CLOCK		0x03	/* clock driver		*/
#define IOC_CHIPSET		0x04	/* chipset driver	*/
#define IOC_CHIP		0x04	/* chipset driver	*/
#define IOC_GRAPHICS		0x05	/* graphic driver	*/
/*				0x06	   reserved		*/
/*				0x07	   reserved		*/
#define	IOC_MULTI		0xff	/* multi-subsystem-commands */

/*\
|*|	A writing ioctl() sends data to the kernel, a reading gets something
|*|	back. 
\*/

/*\
|*|	ioctls for the graphics device
\*/

#define	IOC(subs,nr)		  _IO(IOC_ ## subs, nr) 
#define	IOR(subs,nr,data)	 _IOR(IOC_ ## subs, nr, data)
#define	IOW(subs,nr,data)	 _IOW(IOC_ ## subs, nr, data)
#define	IRW(subs,nr,data)	_IOWR(IOC_ ## subs, nr, data)

/*\
|*|	driver
\*/

#define	DRIVER_GETINFO		IOR(DRIVER, 0x00, struct kgi_driver)
#define	DRIVER_SETINFO		IOW(DRIVER, 0x00, struct kgi_driver)

#define	DRIVER_STRATEGY		IOC(DRIVER, 0x01)

#define	DRIVER_TESTDRIVER	IOC(DRIVER, 0x10)
#define	DRIVER_TESTMODE		IRW(DRIVER, 0x11, ggi_mode)

#define	DRIVER_SETGRAPHMODE	IRW(DRIVER, 0x12, ggi_mode)
#define	DRIVER_GETGRAPHMODE	IOR(DRIVER, 0x13, ggi_mode)
#define	DRIVER_GRAPHICS		IOC(DRIVER, 0x14)

#define	DRIVER_TEXT		IOC(DRIVER, 0x17)
#define	DRIVER_MMIO_OFFSET	IOC(DRIVER, 0x18)

#define	DRIVER_RAMDAEMON_INSTALL	IOC(DRIVER, 0x20)
#define	DRIVER_RAMDAEMON_ASK	IRW(DRIVER, 0x21, ramdaemon_shm)
#define	DRIVER_RAMDAEMON_CPY	IOW(DRIVER, 0x22, ramdaemon_shm)

/*\
|*|	chipset
\*/

#define	CHIP_GETINFO		IOR(CHIP, 0x00, struct kgi_chipset)
#define	CHIP_SETINFO		IOW(CHIP, 0x00, struct kgi_chipset)

#define	CHIP_SETVISFRAME	IOW(CHIP, 0x10, ggi_coord)
#define	CHIP_GETVISFRAME	IOR(CHIP, 0x10, ggi_coord)
#define	CHIP_SETSPLITLINE	IOC(CHIP, 0x11)
#define	CHIP_GETSPLITLINE	IOC(CHIP, 0x12)

#define	CHIP_SETDEFFONT		IOW(CHIP, 0x13, struct kgi_font)
#define	CHIP_GETDEFFONT		IRW(CHIP, 0x13, struct kgi_font)
#define	CHIP_SETFONTCHAR	IOW(CHIP, 0x14, ggi_bm_char)
#define	CHIP_GETFONTCHAR	IRW(CHIP, 0x14, ggi_bm_char)

#define CHIP_SETPAGE		IOW(CHIP, 0x20, int)
#define CHIP_SET16COL		IOW(CHIP, 0x21, int)
#define	CHIP_SETDRAWINFO	IOW(CHIP, 0x22, ggi_drawinfo *) 

/*\
|*|	ramdac
\*/
struct clut256 { ggi_color clut[256]; };	/* !!! temporary ! */

#define	RAMDAC_GETINFO		IOR(RAMDAC, 0x00, struct kgi_ramdac)
#define	RAMDAC_SETINFO		IOW(RAMDAC, 0x00, struct kgi_ramdac)

#define	RAMDAC_SETCLUT		IOW(RAMDAC, 0x10, struct clut256)	/* FIXME !!! */
#define	RAMDAC_GETCLUT		IRW(RAMDAC, 0x11, struct clut256)	/* FIXME !!! */

/*\
|*|	monitor
\*/

#define	MONITOR_GETINFO		IOR(MONITOR, 0x00, struct kgi_monitor)
#define	MONITOR_SETINFO		IOW(MONITOR, 0x01, struct kgi_monitor)

#define	MONITOR_GETNUMPRESET	IOC(MONITOR, 0x10)
#define	MONITOR_GETFREEPRESET	IOC(MONITOR, 0x11)
#define	MONITOR_SETPRESETMODE	IOW(MONITOR, 0x12, struct kgi_preset)
#define	MONITOR_GETPRESETMODE	IRW(MONITOR, 0x12, struct kgi_preset)
#define	MONITOR_CLEARPRESET	IOW(MONITOR, 0x13, struct kgi_preset)

/*\
|*|	graphics
\*/

struct pnt  { int x,y; int col; };		/* !!! temporary ! */
struct box  { int x,y,w,h; };			/* !!! temporary ! */
struct fbox { int x,y,w,h; void *content; };	/* !!! temporary ! */
struct cbox { int x,y,w,h,x2,y2; };		/* !!! temporary ! */
struct line { int x,y,x2,y2;};			/* !!! temporary ! */
struct hlin { int x,y,w; };			/* !!! temporary ! */
struct fhli { int x,y,w; void *content; };	/* !!! temporary ! */
struct circ { int x,y,r; };			/* !!! temporary ! */
typedef struct {int fg_color;			/* !!! temporary ! */
		int bg_color;
		int drawmode; }      ggi_drawinfo;

typedef struct ggi_suggest {
	int	id;
	char	name[256];
	char	args[256];
} ggi_suggest;

/* Everything before 0x10 is a `control' function - choose wisely! */
#define GRAPHICS_GETSUGGEST	IRW(GRAPHICS, 0x01, ggi_suggest)


/* Everything >= 0x10 is a drawing functions */
#define	GRAPHICS_DRAWPIXEL	IOW(GRAPHICS, 0x10, ggi_coord) 
#define	GRAPHICS_PUTPIXEL	IOW(GRAPHICS, 0x11, struct pnt) 
#define	GRAPHICS_GETPIXEL	IOW(GRAPHICS, 0x12, struct pnt) 
#define	GRAPHICS_DRAWHLINE	IOW(GRAPHICS, 0x14, struct hlin) 
#define	GRAPHICS_PUTHLINE	IOW(GRAPHICS, 0x15, struct fhli) 
#define	GRAPHICS_GETHLINE	IOW(GRAPHICS, 0x16, struct fhli) 
#define	GRAPHICS_DRAWLINE	IOW(GRAPHICS, 0x17, struct line) 
#define	GRAPHICS_DRAWVLINE	IOW(GRAPHICS, 0x18, struct hlin) 
#define	GRAPHICS_PUTVLINE	IOW(GRAPHICS, 0x19, struct fhli) 
#define	GRAPHICS_GETVLINE	IOW(GRAPHICS, 0x1a, struct fhli) 
#define	GRAPHICS_DRAWBOX	IOW(GRAPHICS, 0x20, struct box) 
#define	GRAPHICS_GETBOX		IOW(GRAPHICS, 0x21, struct fbox) 
#define	GRAPHICS_PUTBOX		IOW(GRAPHICS, 0x22, struct fbox) 
#define	GRAPHICS_FILLSCREEN	IOC(GRAPHICS, 0x23) 
#define	GRAPHICS_COPYBOX	IOW(GRAPHICS, 0x24, struct cbox) 
#define	GRAPHICS_DRAWCIRCLE	IOW(GRAPHICS, 0x30, struct circ) 

/*\
|*|	multi-commands. These could be supported in various parts
|*|	of the driver. Powersaving is such a thing.
|*|	The kernel driver will ask all subsystems in turn and accept
|*|     the first one returning EOK. If none returns EOK, an error
|*|	code is returned.
\*/

enum powersave  { PWR_ON=0,PWR_STANDBY,PWR_SUSPEND,PWR_OFF };

#define	MULTI_POWERSAVE		IOW(MULTI, 0x08, enum powersave)

/*\
|*|	The various mmap'ing modes. This is given to the mmap call
|*|	as the offset parameter which is otherwise unused.
|*|	More modes are likely to be added. Especially MMAP_ACCEL.
\*/


/* Well known mmap-area types :
*/

#define	MMAP_PER_REGION_TYPE	0x00000000
#define	MMAP_PER_REGION_INDEX	0x80000000

#define	MMAP_ENDIAN		0x40000000
#define	MMAP_LITTLE_ENDIAN	0x00000000
#define	MMAP_BIG_ENDIAN		0x40000000

#define	MMAP_TYPE		0x0F000000
#define	MMAP_TYPE_MMIO		0x00000000
#define	MMAP_TYPE_ACCEL		0x01000000
#define	MMAP_TYPE_GC		0x02000000

#define	MMAP_INDEX		0x0000FFFF
#define	MMAP_PUBLIC		0x00000000
#define	MMAP_PRIVATE		0x00008000
#define	MMAP_FRAMEBUFFER	0x00000001


/* These are for accessing kernel-mode structures
*/
#define MMAP_DRAWINFO		0x0100	/* struct drawinfo writeable by app */

/*\
|*|	ioctl transfer
|*|
|*|	The data transfer is done as follows: 
|*|	(IO_BUF, OUT_BUF and IO_CNT are file local now, may change!)
|*|
|*|	  - The kernel driver checks if the IOC_IN flag in cmd is set. If so,
|*|	    the number of bytes to copy from arg is determined from cmd and
|*|	    copied to _io_buf. _out_buf is set to NULL.
|*|
|*|	  - Then kgi_subsystem_ioctl is invoked according to the subsystem 
|*|	    coded in cmd. This performs the action according to the request in
|*|	    cmd. If output is requested, _out_buf is set to the 'result' buffer
|*|	    and _io_cnt to the number of bytes to copy from there. Note that
|*|	    local variables will be out of scope after return. Use _io_buf then.
|*|
|*|	  - After kgi_subsystem_ioctl has returned, cmd is checked for the 
|*|	    IOC_OUT flag to be set. If so (and _out_buf and _io_cnt are valid),
|*|	    _io_cnt bytes are copied from _out_buf to arg. 
|*|	    
|*|	  - The value returned by kgi_subsystem_ioctl is returned.
\*/

#endif	/* #ifdef _ggi_commands_h */
