/* 
 *  Graphics library for GGI.
 * 
 *  Copyright (C) 1997 Jason McMullan - jmcc@cs.cmu.edu
 * 
 *  If you do any modifications, I would like you to send diffs to me
 *  to allow for collecting a more and more complete set of drivers and
 *  to improve existing ones.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * ----------------------------------------------------------------------
 * 
 *	$Log: conf.c,v $
 *	Revision 1.3  1997/11/30 13:55:55  becka
 *	Threads fixes, Extension handling.
 *	
 *	Revision 1.2  1997/11/15 01:07:58  willie
 *	Changed GPL to LGPL License
 *	
 *	Revision 1.1  1997/10/12 04:53:38  tfries
 *	new subdir
 *	
 *	Revision 1.4  1997/05/16 05:12:11  ezrec
 *	Major: Event support, and updated ggilib shim. Xggi 0.0.8 binaries should
 *		work now, with a minor caveat on mouse buttons..
 *	
 *	Revision 1.3  1997/05/07 15:47:35  ezrec
 *	Minor: Fixed bug with `.root' parsing
 *
 *	Revision 1.2  1997/04/17 05:08:10  ezrec
 *	Major: 	New ggiOpen() semantics ie ggiOpen("display-KGI:/dev/graph3",NULL)
 *		First attempt at linear_1 frame buffer
 *		Moved thread support `to the side'
 *		Added DPRINT() support
 *		Added display-KGI driver
 *
 *	Revision 1.1  1997/03/25 04:54:09  ezrec
 *	Major: libggi now handles libggi.conf files, IBM vga driver suggests
 *	      default ioctl RAMDAC DLL, default ioctl RAMDAC DLL support,
 *	      -Wall added to the libggi-dynamic Makefiles.
 *
 * 
 * ----------------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#include "internal.h"

ggi_dlconfig *_ggiMasterConfig=NULL;

int _ggiLoadConfig(const char *file)
{
	FILE *inf;
	char root[2048],line[2048],buff[2048],*cp,*bp;
	char *name,*fname;
	ggi_dlconfig *dltmp,*dlconf;
	int i;
	static int incl=0;
	root[0]=0;

	inf=fopen(file,"r");
	if (inf==NULL)
		return -1;

	dlconf=_ggiMasterConfig;

	while (!feof(inf)) {
		fgets(line,2048,inf);
		cp=line;

		while (isspace(*cp)) cp++;
		if (*cp==0 || *cp=='#')
			continue;
	
		if (strncmp(cp,".root:",6)==0) {
			cp+=6;
			while (isspace(*cp)) cp++;
			name=cp;
			while (*cp && !isspace(*cp)) cp++;
			strncpy(root,name,cp-name);
			root[cp-name]=0;
			continue;
		}
		if (strncmp(cp,".include",8)==0) {
			cp+=8;
			while (isspace(*cp)) cp++;
			name=cp;
			while (*cp && !isspace(*cp)) cp++;
			strncpy(buff,name,cp-name);
			buff[cp-name]=0;
			incl++;
			if (incl<20) {
				_ggiMasterConfig=dlconf;
				_ggiLoadConfig(buff);
				dlconf=_ggiMasterConfig;
			}
			else {
				fprintf(stderr,"libggi: .include recursion "
						"too deep\n");
			}

		}	
		bp=buff;
		while (!isspace(*cp)) *(bp++)=*(cp++);
		*(bp)=0;
		name=strdup(buff);
			
			
		while (isspace(*cp)) cp++;
		if (*cp==0 || *cp=='#') {
			free(name);
			continue;
		}
		bp=buff;
		while (!isspace(*cp)) *(bp++)=*(cp++);
		*(bp)=0;
		if (buff[0]!='/') {
			i=strlen(buff)+strlen(root)+2;
			fname=(char *)malloc(i);
			strcpy(fname,root);
			strcat(fname,"/");
			strcat(fname,buff);
		} else
			fname=strdup(buff);

		/* Add the config entry....
		 */	
		dltmp=(ggi_dlconfig *)malloc(sizeof(ggi_dlconfig));
		dltmp->name=name;
		dltmp->filename=fname;
		dltmp->version=NULL;
		dltmp->next=dlconf;
		dlconf=dltmp;

		do {	
			ggi_string_list *vtmp;

			while (isspace(*cp)) cp++;
			if (*cp==0 || *cp=='#') 
				break;
			bp=buff;
			while (!isspace(*cp)) *(bp++)=*(cp++);
			*(bp)=0;
			vtmp=(ggi_string_list *)malloc(sizeof(ggi_string_list));
			vtmp->str=strdup(buff);
			vtmp->next=dltmp->version;
			dltmp->version=vtmp;
		} while (*cp!=0);
			
	}
	fclose(inf);

	_ggiMasterConfig=dlconf;

	return 0;
}


/* Takes a ggi_dlconfig, lib name, and a version. Returns the
 * closest match (Ha!)
 */
const char *_ggiMatchConfig(const char *name,const char *ver)
{
	ggi_dlconfig *dlc,*dtmp,*best=NULL;
	ggi_string_list *vtmp;

	dlc=_ggiMasterConfig;

	for (dtmp=dlc;dtmp!=NULL;dtmp=dtmp->next) {
		if (strcmp(name,dtmp->name)!=0)
			continue;

		if (dtmp->version==NULL) {
			best=dtmp;
			continue;
		}

		for (vtmp=dtmp->version;vtmp!=NULL;vtmp=vtmp->next) 
			if (strcmp(ver,vtmp->str)==0) {
				return dtmp->filename;
			}
	}

	return (best==NULL) ? NULL : best->filename;
}
			

		
