/* 
   Graphics library for GGI. Pixels.

   Copyright (C) 1997 Jason McMullan - jmcc@cs.cmu.edu

   If you do any modifications, I would like you to send diffs to me
   to allow for collecting a more and more complete set of drivers and
   to improve existing ones.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <string.h>

#include "ggi-dl.h"

/*****************************************/
/* 4-bpp screen->screen rectangular copy */
/*****************************************/

int GGIcopybox(ggi_visual *vis,int x,int y,int w,int h,int nx,int ny)
{
	uint8 *obox,*nbox;
	int linew=VIS_FB_WIDTH(vis) >> 1; /* in bytes, I assume */
	int line;
	int left, right;
	
	CHECKXYWH(vis,x,y,w,h);
	CHECKXYWH(vis,nx,ny,w,h);
	
	/* if x is odd, left pixel dangles. */
	/* if x is odd and w is even, right dangles. */
	/* if x is even and w is odd, right dangles. */

	left  = (x & 0x01);
	right = (x ^ w) & 0x01;
	w -= left + right;
	
	if (ny < y) {

		obox=(uint8 *)(VIS_FB_LINEAR(vis))+y*linew+(x>>1);
		nbox=(uint8 *)(VIS_FB_LINEAR(vis))+ny*linew+(nx>>1);
		if (left) {
			nbox++; obox++;
		}
		for (line=0;line<h;line++,obox+=linew,nbox+=linew) {
			
			if (left)
				*(nbox-1) = (*(nbox-1) & 0xF0) | *(obox-1);
			
			memmove(nbox,obox,w>>1);
			
			if (right)
				*(nbox+w) = (*(nbox+w) & 0x0F) | (*(obox+w) << 4);
		}
		
	} else {
		obox=(uint8 *)(VIS_FB_LINEAR(vis))+(y+h-1)*linew+(x>>1);
		nbox=(uint8 *)(VIS_FB_LINEAR(vis))+(ny+h-1)*linew+(nx>>1);
		if (left) {
			nbox++; obox++;
		}
		for (line=0;line<h;line++,obox-=linew,nbox-=linew) {

			if (left)
				*(nbox-1) = (*(nbox-1) & 0xF0) | *(obox-1);
			
			memmove(nbox,obox,w>>1);
			
			if (right)
				*(nbox+w) = (*(nbox+w) & 0x0F) | (*(obox+w) << 4);
		}
	}
	return 0;
}
