/* 
   Graphics library for GGI. 
   Generic RAMDAC via IOCTL driver

   Copyright (C) 1997 Jason McMullan - jmcc@cs.cmu.edu

   If you do any modifications, I would like you to send diffs to me
   to allow for collecting a more and more complete set of drivers and
   to improve existing ones.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <math.h>
#include <sys/ioctl.h>

#ifdef _AIX
#include <sys/types.h>
#include <unistd.h>
#endif

#include "ggi-dl.h"
#include <ggi/ggi_commands.h>

extern ggi_color *_GGIcolormap;
extern int _GGIbpc;
extern int _GGInocols;

/* Do closest-match-mapping. I use a || x ||_1 norm here. This is faster
 * and the measure for "closest" istn't well defined for RGB anyway.
 */
ggi_pixel GGImapcolor(ggi_visual *vis,ggi_color *col)
{
	int closest=0;
	int i;
	unsigned short closedist=0xffff,dist,d;
	
#undef ABS
#define ABS(a,b) ( (a>b) ? a-b : b-a )
		for (i=0;i<_GGInocols;i++) {
			dist=ABS(col->r,_GGIcolormap[i].r);
			if (dist >= closedist) continue;
			d   =ABS(col->g,_GGIcolormap[i].g);
			if (d>dist) dist=d;
			if (dist >= closedist) continue;
			d   =ABS(col->b,_GGIcolormap[i].b);
#undef ABS
			if (d>dist) dist=d;
			if (dist >= closedist) continue;
			closest=i;
			if (dist==0) break;
			closedist=dist;
		}
		DPRINT("colmap: chose %x,%x,%x (%d) for %x,%x,%x.\n",
			_GGIcolormap[closest].r,
			_GGIcolormap[closest].g,
			_GGIcolormap[closest].b,closest,
			col->r,col->g,col->b
			);
		return closest;
}

/* Map pixel to color
 */
int GGIunmappixel(ggi_visual *vis,ggi_uint pixel,ggi_color *col)
{
	if (col==NULL || pixel>=_GGInocols)
		return -1;

	*col=_GGIcolormap[pixel];

	return EOK;	
}

int GGIsetpalvec(ggi_visual *vis,int start,int len,ggi_color *colormap)
{
	if (colormap==NULL || start+len >_GGInocols)
		return -1;

	memcpy(_GGIcolormap+start,colormap,len*sizeof(ggi_color));

	return ioctl(vis->fd,(int)RAMDAC_SETCLUT,_GGIcolormap);
}

int GGIgetpalvec(ggi_visual *vis,int start,int len,ggi_color *colormap)
{
	if (colormap==NULL || start+len >_GGInocols)
		return -1;

	memcpy(colormap,_GGIcolormap+start,len*sizeof(ggi_color));

	return 0;
}
  
