/* 
   Graphics library for GGI.

   Copyright (C) 1995 Andreas Beck - becka@hp.rz.uni-duesseldorf.de
   Copyright (C) 1997 Jason McMullan - jmcc@cs.cmu.edu

   If you do any modifications, I would like you to send diffs to me
   to allow for collecting a more and more complete set of drivers and
   to improve existing ones.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "ggi-dl.h"


/* Cross blitting of a rectangular area. FIXME ! FIXME ! FIXME !
 * There are many unimplemented things here.
 */
int GGIcrossblit(ggi_visual *src,int sx,int sy,int sw,int sh,
		 ggi_visual *dst,int dx,int dy,int dw,int dh)
{
	ggi_color thecol;
	ggi_uint pixel,lastpixel,lasttrans;
	int errx,erry,ddx,ssx,ty,tx;
	
	lasttrans=0;	/* Keep compiler quiet. */
	ggiGetPixel(src,sx,sy,&lastpixel);lastpixel++; /* assure safe init */

	/* Clipping is done here via the fallback functions.
	   For some cases we could do a pre-clip, but we need to take care
	   about the right error-terms.
	   At least we could do some paranoia-clipping a in drawcircle.
	   Anybody add this, please.
	 */

	if (sw==dw&&sh==dh) {	/* NOSTRETCH */

		sh+=sy;sw+=sx;ssx=sx;ddx=dx;
		for(;sy<sh;sy++,dy++) {
			sx=ssx;dx=ddx;
			for(;sx<sw;sx++,dx++) {
				ggiGetPixel(src,sx,sy,&pixel);
				if (pixel!=lastpixel) {
					ggiUnmapPixel(src,lastpixel=pixel,&thecol);
					lasttrans=ggiMapColor(dst,&thecol);
				}
				ggiPutPixel(dst,dx,dy,lasttrans);
			}
		}

	} else {		/* STETCH */

		ddx=dx;ssx=sx;erry=(dh-sh)>>1;ty=sy+sh;tx=sx+sw;
	
		/* The basic stretching core. Please speed me up. Teunis ? */
		for(;sy<ty;sy++) {
			erry+=dh;
			if (erry<=0) continue;

			dx=ddx;sx=ssx;errx=0;
			for(;sx<tx;sx++) {
				errx+=dw;
				if (errx<=0) continue;
				ggiGetPixel(src,sx,sy,&pixel);

				if (pixel!=lastpixel) {
					ggiUnmapPixel(src,lastpixel=pixel,&thecol);
					lasttrans=ggiMapColor(dst,&thecol);
				}
				
				while(errx>0) {
					ggiPutPixel(dst,dx++,dy,lasttrans);
					errx-=sw;
				}
			}
			do{
				dy++;erry-=sh;
				ggiCopyBox(dst,ddx,dy-1,dw,1,ddx,dy);
			} while(erry>0);
		}
	}		/* STETCH */
	return EOK;
}
