
/* This module takes care of setting up a default EvStack environment.
 */

#include "evstack.h"
#include "parsearg.h"

#undef MEGA_DEBUG

static void dump_stack(evstack *stack,char *class)
{
	int i;
	char buffer[256];
	
	printk("$ cat /proc/%s/%08x\n",class,stack->id);
	for (i=0;;i++) {
		if (!stack->ops->read || stack->ops->read(stack,i,sizeof(buffer),buffer)<0)
		  break;
		printk("%s\n",buffer);
	}
}

static evstack *create_and_attach_stack(evstack *termstack,const char *name)
{
	int id;
	evstack *stk;
	char buffer[128];

	id=ev_lookup_class(name);

	if (id<0) {
		printk(KERN_ERR "Can't find stack class \"%s\"\n",
			name);
		return NULL;
	}

	/* Tell which stack to send info to */
	stk=ev_create_stack(id);
	sprintf(buffer,"attach %s",par_stack_to_string(stk));

	termstack->ops->write(termstack,buffer);

	return stk;
}

static evstack *clone_and_attach_stack(evstack *termstack,evstack *stack)
{
	evstack *stk;
	char buffer[128];

	/* Tell which stack to send info to */
	stk=ev_clone_stack(stack);
	sprintf(buffer,"attach %s",par_stack_to_string(stk));

	termstack->ops->write(termstack,buffer);

	return stk;
}

void attach_stack(evstack *termstack,evstack *stk)
{
	char buffer[128];

	/* Tell which stack to send info to */
	sprintf(buffer,"attach %s",par_stack_to_string(stk));
	termstack->ops->write(termstack,buffer);

	return;
}

int opt_d=0;

static evstack *termstack,*vtswitch,*head,*temp,*selection;

static evstack *loadkeys,*parser,*devevent,*devtty,*debug,*vgascroll;

int init_module(void)
{
	ggi_event *page;
	ggi_key_event key;
	int i,pagesize;
	char buffer[256];

/* Create a termstack */
	termstack=ev_create_stack(ev_lookup_class("termstack"));
	if (termstack==NULL) {
		printk(KERN_ERR "Can't create termstack !\n");
		return -1;
	}

	/* Populate the termstack */
#ifdef LOADKEYS_IN_STACK
	loadkeys=create_and_attach_stack(termstack,"loadkeys");
#endif
	parser=create_and_attach_stack(termstack,"xterm");

	devtty=create_and_attach_stack(termstack,"devtty");

	attach_stack(termstack,parser);

	vgascroll=create_and_attach_stack(termstack,"vgascroll");

	dump_stack(termstack,"termstack");
	
/* Create the VtSwitcher */
	vtswitch=ev_create_stack(ev_lookup_class("vtswitch"));
	if (vtswitch==NULL) {
		printk(KERN_ERR "Can't create vtswitch !\n");
		return -1;
	}

	/* Attach TermStacks to VT#0 */
	sprintf(buffer,"attach %s %d",par_stack_to_string(termstack),0);
	vtswitch->ops->write(vtswitch,buffer);
	sprintf(buffer,"attach %s %d",par_stack_to_string(termstack),2);
	vtswitch->ops->write(vtswitch,buffer);
	dump_stack(vtswitch,"vtswitch");

/* Create the Head */
	head=ev_create_stack(ev_lookup_class("head"));
	if (head==NULL) {
		printk(KERN_ERR "Can't create head!\n");
		return -1;
	}

#ifndef LOADKEYS_IN_STACK
	loadkeys=create_and_attach_stack(head,"loadkeys");
#endif
	/* attach the vtswitcher to the head */
	sprintf(buffer,"attach %s",par_stack_to_string(vtswitch));
	head->ops->write(head,buffer);

	/* attach the selection tool to the head */
	selection=ev_create_stack(ev_lookup_class("selection"));
	sprintf(buffer,"attach %s",par_stack_to_string(selection));
	head->ops->write(head,buffer);

	/* attach an event-device to the head */
	devevent=create_and_attach_stack(head,"devevent");

	/* if asked for, attach the debugger to the head */
	if (opt_d) {
		debug=ev_create_stack(ev_lookup_class("debug"));
		sprintf(buffer,"attach %s",par_stack_to_string(debug));
		head->ops->write(head,buffer);
	}

	dump_stack(head,"head");

	page=ev_evpage_alloc();
	page->size=0;
	pagesize=EVPAGESIZE;

	key.type=evKeyPress;
	key.code=5;
	key.size=sizeof(key);
	
	/* Add the event to the page */
	ev_evpage_add(page,&pagesize,(ggi_event *)(&key));

	{	DataVtSwitch dss;
		char outbuf[256];

		dss.newvt=0;
		build_cmd_event(outbuf,CmdVtSwitch,&dss,sizeof(dss));
		DEBUG2("VtSwitch %d",val);
		ev_evpage_add(page,&pagesize,(ggi_event *)&outbuf);
	}

	ev_send(page,&pagesize,NULL,head);
                                                	
#if 0
	{
		DataTtyWriteChar evdata;
		int x;
		evdata.size=80;
		for(x=0;x<80;x++)
			evdata.data[x]='A';

		build_cmd_event(buffer,CmdTtyOutWriteChar,&evdata,84);
		for(x=0;x<125;x++)
		{
			ev_evpage_add(page,&pagesize,(ggi_event *)buffer);
			head->ops->handler(head,page,&pagesize);
			page->size=0;
			pagesize=EVPAGESIZE;
		}
	}
#endif

	ev_evpage_free(page);	        	

	return 0;
}

void cleanup_module(void)
{
	ev_destroy_stack(head);
	if (opt_d) {
		ev_destroy_stack(debug);
	}
	ev_destroy_stack(devtty);
	ev_destroy_stack(selection);
	ev_destroy_stack(vtswitch);
	ev_destroy_stack(termstack);
	ev_destroy_stack(vgascroll);
	ev_destroy_stack(devevent);
	ev_destroy_stack(parser);
	ev_destroy_stack(loadkeys);
}
