#include <fcntl.h>
#include <unistd.h>

#undef DEBUG
#ifdef DEBUG
#define DEBUG_LEVEL 2
#endif
#include <ggi/libggi.h>
#include "evstack.h"

typedef struct {
	int rdpipe[2];
	int wrpipe[2];
} pipetty_state ;

pipetty_state default_state={ {-1,-1}, {-1,-1} };

static int  pipetty_use=0;

static int  pipetty_open(evstack *blank);
static int  pipetty_close(evstack *slf);
static void pipetty_handler(evstack *slf,ggi_event *ev,int *left);
static int  pipetty_uses(evstack *slf);
static int  pipetty_write(evstack *slf,char *str);
static int  pipetty_read(evstack *slf,int lineno,int maxlen,char *str);

static evstack_operations pipetty_ops={
	pipetty_open,
	pipetty_close,
	pipetty_handler,
	pipetty_uses,
	pipetty_write,
	pipetty_read
};

static int pipetty_open(evstack *blank)
{
	pipetty_state *ps;

	sprintf(blank->name,"PipeTTY v1.0 (instance %.4x)",
		blank->id&EV_ID_MASK);

	blank->maintainer="becka@sunserver1.rz.uni-duesseldorf.de";

	ps=blank->private=ev_malloc(sizeof(pipetty_state));
	memcpy(blank->private,&default_state,sizeof(pipetty_state));

	pipetty_use++;

	return 0;
}

static int pipetty_close(evstack *slf)
{
	pipetty_use--;

	return 0;
}

	
/* A sample tty module
 */
static void pipetty_handler(evstack *slf,ggi_event *ev,int *left)
{ 
	ggi_cmd_event *evdat;
	pipetty_state *ps=slf->private;
	char outchar[256];
	DataTtyWriteChar fromshell;

	for(;ev->size;ev=ev_evpage_next(ev))       /* Loop through all events in page */
	{
		/* We are interested in sending cooked data to the user. */
		switch(ev->any.type)
		{ 
		  case evCommand:
		  	evdat=(ggi_cmd_event *)ev;
		  	switch(evdat->code)
		  	{
		  	  case CmdTtyInWriteChar:
		  	  	ev_consume(ev);
				DEBUG1("got event %x.\n",evdat->code);
				{ DataTtyWriteChar *dt=(DataTtyWriteChar *)evdat->data;
				  DEBUG1("%d chars :%*.*s.\n",dt->size
				  	  ,dt->size,dt->size,dt->data);
				  /* now we send it to usermode */
				  write(ps->rdpipe[1],dt->data,dt->size);

				  /* For testing purposes we simply echo it. */
				  build_cmd_event(outchar,CmdTtyOutWriteChar,dt,dt->size+sizeof(dt->size));

				  DEBUG1("sending echo command '%c'.\n",dt->data[0]);

	                          TRACE1({ int x;
	                          for(x=0;x<20;x++)
        	                  	printk("%2x ",*(((unsigned char *)(outchar))+x));
	                          printk(")\n"); }
	                          );
                                                                                                  
				  ev_evpage_add(ev,left,(ggi_event *)&outchar);
				} break;
		  	  default:
		  	  	break;
			}
			break;
		  default: 
		  	break;
		}
	}
	fromshell.size=read (ps->wrpipe[0],fromshell.data,64);
	if (fromshell.size>0)
	{	build_cmd_event(outchar,CmdTtyOutWriteChar,&fromshell,fromshell.size+sizeof(fromshell.size));
		ev_evpage_add(ev,left,(ggi_event *)&outchar);
	}
}

static int pipetty_uses(evstack *stack)
{
	return pipetty_use;
}

static int pipetty_write(evstack *stack, char *str)
{
	return -EINVAL;
}
static int pipetty_read(evstack *slf,int lineno,int maxlen,char *str)
{
	if (lineno==0) 
	{
		strcpy(str,"No info available."); return 0;
	}
	return -EINVAL;
}


void init_pipetty(void)
{ 
	char buffer[256];
	
	pipe(default_state.rdpipe);
	pipe(default_state.wrpipe);
	fcntl(default_state.wrpipe[0],F_SETFL,O_NONBLOCK);
	sprintf(buffer,"bash <&%d >&%d 2>&%d &",default_state.rdpipe[0],
		default_state.wrpipe[1],default_state.wrpipe[1]);
	system(buffer);

	ev_register_class("pipetty",&pipetty_ops);
	printk("Registered pipetty.\n");
}
