enum {	cursor, id, status, 
	mouseup, 
	mousedown1, mousedown2, mousedown3
};

static ggi_color colormap[16] =
{
	{0x0000,0x0000,0x0000} /* BLACK     */,
	{0x8000,0x0000,0x0000} /* RED       */,
	{0x0000,0x8000,0x0000} /* GREEN     */,
	{0x8000,0x8000,0x0000} /* BROWN     */,
	{0x0000,0x0000,0x8000} /* BLUE      */,
	{0x8000,0x0000,0x8000} /* MAGENTA   */,
	{0x0000,0x8000,0x8000} /* CYAN      */,
	{0x8000,0x8000,0x8000} /* LIGHTGRAY */,
	{0x4000,0x4000,0x4000} /* DARKGRAY  */,
	{0xffff,0x0000,0x0000} /* LIGHTRED  */,
	{0x0000,0xffff,0x0000} /* LIGHTGREEN*/,
	{0xffff,0xffff,0x0000} /* YELLOW    */,
	{0x0000,0x0000,0xffff} /* LIGHTBLUE */,	
	{0xffff,0x0000,0xffff} /* LIGHTMAGENTA*/,
	{0x0000,0xffff,0xffff} /* LIGHTCYAN */,	
	{0xffff,0xffff,0xffff} /* WHITE     */
};

enum 
{	COL_BLACK,	COL_RED,	COL_GREEN,	COL_BROWN,
	COL_BLUE,	COL_MAGENTA,	COL_CYAN,	COL_LIGHTGRAY,
	COL_DARKGRAY,	COL_LIGHTRED,	COL_LIGHTGREEN,	COL_YELLOW,
	COL_LIGHTBLUE,	COL_LIGHTMAGENTA,COL_LIGHTCYAN,	COL_WHITE
};

static const char *funckey[]={
	"\033[[A",
	"\033[[B", 
	"\033[[C", 
	"\033[[D", 
	"\033[[E", 
	"\033[17~", 
	"\033[18~", 
	"\033[19~", 
	"\033[20~", 
	"\033[21~", 
	"\033[23~", 
	"\033[24~", 
	"\033[25~", 
	"\033[26~", 
	"\033[28~", 
	"\033[29~", 
	"\033[31~", 
	"\033[32~", 
	"\033[33~", 
	"\033[34~", 
	"\033[1~", 
	"\033[2~", 
	"\033[3~", 
	"\033[4~", 
	"\033[5~", 
	"\033[6~", 
	"\033[M", 
	"\033[P" 
};

#define FUNCKEYNUM (sizeof(funckey)/sizeof(funckey[0]))

#define	DEFAULT_FG	(colormap[COL_WHITE])
#define	DEFAULT_BG	(colormap[COL_BLACK])

#define	TEST_FLAG(var,x)	(var &  (1 << x))
#define	SET_FLAG(var,x)		var |=  (1 << x)
#define	CLEAR_FLAG(var,x)	var &= ~(1 << x)
#define	TOGGLE_FLAG(var,x)	var ^= (1 << x)

#define	TRANS_FLAG(to,from,x)	(to = (to & ~(1 << x)) | (from &  (1 << x)))

#define	MAX_COLUMNS	160
#define	TAB_STOPS	MAX_COLUMNS/(8*sizeof(ulong))

enum modes
{
	smooth_scroll,	/* smooth(slow) scroll		*/
	reverse_video,	/* black on white		*/
	origin,		/* cursor origin mode		*/
	auto_wrap,	/* automatic line wrap		*/

	auto_repeat,	/* repeat chars automatically	*/
	reverse_wrap,	/* allow reverse wrapping	*/
	alt_screen,	/* use alternate screen		*/
	show_cursor,	/* show/hide cursor		*/

	scroll_last_mode /* NOTE: there *must* not be more than 32 modes! */
};

enum xterm_modes
{
	display_ctrl = scroll_last_mode,		/* display control characters	*/
	toggle_meta,			/* set meta bit 0x80		*/
	utf8,				/* do utf8 decoding		*/
	applic_key,			/* keypad in application mode	*/

	cursor_key,			/* DECCKM - cursor key mode	*/
	column132,			/* 132 column mode		*/
	x10_mouse,			/* x10 compatible mouse report	*/
	textronixmode,			/* enter textronix mode		*/

	allow_132mode,			/* allow switch to 132 columns	*/
	curses_fix,			/* allow hack for curses	*/
	margin_bell,			/* margin bell			*/
	logging,			/* do logging			*/

	report_mouse,			/* report mouse			*/
	track_mouse,			/* report & hilite track mouse	*/
	ansi,				/* DECANM - ANSI mode		*/
	newline,			/* ENTER sends CR/CRLF		*/

	insert,				/* ANSI insert mode		*/
	meta_esc,			/* META chars with ESC-prefix	*/

	xterm_last_mode
};

enum flags
{
	need_wrap,	/* cursor at rightmost column	*/
	scrolled,	/* ORIGIN or OFFSET modified	*/
	no_hardscroll,	/* small hardware framebuffer	*/
	splitline,	/* use wrapping mode, don't copy*/
	compose,	/* key compose in progress	*/

	scroll_last_flag /* NOTE: there *must* not be more than 32 flags! */
};

enum xterm_flags 
{
	question = scroll_last_flag,	/* ESC?[ sequence?	*/
	num_lock,			/* num_lock key active	*/
	caps_lock,			/* CAPS-shift, locked	*/
	caps_sticky,			/* CAPS-shift, sticky	*/

	xterm_last_flag
};
enum xterm_states
{
	ESnormal,	ESesc,		ESsquare,	ESgetpars,
	ESgotpars,	ESfunckey, 	EShash,		ESset,
	ESpercent,	ESignore
};


#define CTRL_ACTION 0x0d00ff81	/* chars<32 that invoke special action	*/
#define	CTRL_ALWAYS 0x0800f581	/* can not be overridden by DISP_CTRL	*/

#define MAX_NPAR 16

#define	LAT1_MAP	0
#define	GRAF_MAP	1
#define	IBMPC_MAP	2
#define	USER_MAP	3

