/* --------------------------------------------------------------------------
**	Common parsing functions for LibGGI display targets.
** --------------------------------------------------------------------------
**
**	Copyright (C) 1998, The GGI project.
**
**	This program is free software; you can redistribute it and/or
**	modify it under the terms of the GNU Library General Public
**	License as published by the Free Software Foundation; either
**	version 2, or (at your option) any later version.
**
**	You should have received a copy of the GNU Library General
**	Public License along with this program; see the file
**	COPYING.LGPL.  If not, write to the Free Software Foundation,
**	675 Mass Ave, Cambridge, MA 02139, USA.
**
** ---------------------------------------------------------------------------
*/

/* _GGIparsetarget:
 *
 *	Parses a target descriptor out of a string, and returns a
 *	pointer to after the last character of the target.  Handles
 *	bracketized forms appropriately.  Returns NULL if an error
 *	occurs.
 *
 *	For example:
 *
 *		"display-multi:(display-KGI:/dev/head1):display-X\0"
 *			       ^		       ^
 *			       |__str	               |__result
 *
 *		target: "display-KGI\0:/dev/head1"
 */

static char *_GGIparsetarget(char *str, char *target)
{
	int bracketized=0;
	int bracket_count=0;

	if (*str == '(') {
		bracketized=1;
		bracket_count++;
		str++;
	}

	while (*str) {
		if (*str == '(') {
			bracket_count++;
		
		} else if (*str == ')') {
		
			if (bracket_count == 0) {
				fprintf(stderr, "libggi: Bad target "
					"descriptor : unexpected ')'.\n");
				*target = 0;
				return NULL;
			}

			bracket_count--;

			if (bracketized && (bracket_count == 0)) {
				str++;
				break;
			}
		}

		*target++ = *str++;
	}

	*target = 0;

	if (bracket_count > 0) {
		fprintf(stderr, "libggi: Bad target descriptor : "
			"missing ')'.\n");
		return NULL;
	}

	return str;
}
