/* 
   Graphics library for GGI. 
   Palette driver for SVGAlib.

   Copyright (C) 1997 Steve Cheng <elmert@ipoline.com>
   Copyright (C) 1997 Andreas Beck - becka@uni-duesseldorf.de

   If you do any modifications, I would like you to send diffs to me
   to allow for collecting a more and more complete set of drivers and
   to improve existing ones.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "ggi-dl.h"
#include "vgavisual.h"

/* Do closest-match-mapping. I use a || x ||_1 norm here. This is faster
 * and the measure for "closest" istn't well defined for RGB anyway.
 */
ggi_pixel GGImapcolor(ggi_visual *vis,ggi_color *col)
{
	int closest=0;
	int i;
	unsigned short closedist=0xffff,dist,d;
	
#undef ABS
#define ABS(a,b) ( (a>b) ? a-b : b-a )
		for (i=0;i<SVGA_PRIV(vis)->nocols;i++) {
			dist=ABS(col->r,SVGA_PRIV(vis)->colormap[i].r);
			if (dist >= closedist) continue;
			d   =ABS(col->g,SVGA_PRIV(vis)->colormap[i].g);
			if (d>dist) dist=d;
			if (dist >= closedist) continue;
			d   =ABS(col->b,SVGA_PRIV(vis)->colormap[i].b);
#undef ABS
			if (d>dist) dist=d;
			if (dist >= closedist) continue;
			closest=i;
			if (dist==0) break;
			closedist=dist;
		}
		DPRINT("colmap: chose %x,%x,%x (%d) for %x,%x,%x.",
			SVGA_PRIV(vis)->colormap[closest].r,
			SVGA_PRIV(vis)->colormap[closest].g,
			SVGA_PRIV(vis)->colormap[closest].b,closest,
			col->r,col->g,col->b
			);
		return closest;
}

/* Map pixel to color
 */
int GGIunmappixel(ggi_visual *vis,ggi_pixel pixel,ggi_color *col)
{
	if (col==NULL || pixel>=SVGA_PRIV(vis)->nocols)
		return -1;

	*col=SVGA_PRIV(vis)->colormap[pixel];

	return EOK;	
}

int GGIsetpalvec(ggi_visual *vis,int start,int len,ggi_color *colormap) {
	int i;

	if (colormap==NULL || start+len >SVGA_PRIV(vis)->nocols)
		return -1;
                        
	memcpy(SVGA_PRIV(vis)->colormap+start,colormap,len*sizeof(ggi_color)); 

	/* vga_setpalette takes 6-bit r,g,b,
	   so we need to scale ggi_color's 16-bit values. */
	for(i = start; i<start+len; i++)
		vga_setpalette(i, SVGA_PRIV(vis)->colormap[i].r >>10,
				SVGA_PRIV(vis)->colormap[i].g >>10,
				SVGA_PRIV(vis)->colormap[i].b >>10);
	return 0;

}

int GGIgetpalvec(ggi_visual *vis,int start,int len,ggi_color *colormap)
{
	if (colormap==NULL || start+len >SVGA_PRIV(vis)->nocols)
		return -1;

	/* We can use vga_getpalvec here instead */
	memcpy(colormap,SVGA_PRIV(vis)->colormap+start,len*sizeof(ggi_color));

	return 0;
}
