/* ------------------------------------------------------------------------
 *	events.c  - part of the TrueEmu target of GGI.
 * ------------------------------------------------------------------------
 *
 *	Copyright (c) 1998  Andrew Apted  <andrew@ggi-project.org>
 *
 *	This library is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU Library General Public
 *	License as published by the Free Software Foundation; either
 *	version 2, or (at your option) any later version.
 *
 *	This library is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Library General Public License for more details.
 *
 *	You should have received a copy of the GNU Library General
 *	Public License along with this library; see the file
 *	COPYING.LGPL.  If not, write to the Free Software Foundation,
 *	675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * ------------------------------------------------------------------------
 */

#include "ggi-dl.h"
#include "debug.h"

#include "trueemu.h"


static void cycle_dither(TrueemuHook *th)
{
	int old_dith = th->mode & TE_DITHER_MASK;

	th->mode &= ~TE_DITHER_MASK;

	switch (old_dith)
	{
		case TRUEEMU_M_DITHER_0:
			th->mode |= TRUEEMU_M_DITHER_2;
			break;

		case TRUEEMU_M_DITHER_2:
			th->mode |= TRUEEMU_M_DITHER_4;
			break;

		default:
			th->mode |= TRUEEMU_M_DITHER_0;
			break;
	}
}


static void cycle_palette(TrueemuHook *th)
{
	int old_pal = th->mode & TE_PALETTE_MASK;

	th->mode &= ~TE_PALETTE_MASK;

	switch (old_pal)
	{
		case TRUEEMU_M_RGB:
			th->mode |= TRUEEMU_M_CUBE;
			break;

		case TRUEEMU_M_CUBE:
			th->mode |= TRUEEMU_M_PASTEL;
			break;

		case TRUEEMU_M_PASTEL:
			th->mode |= TRUEEMU_M_PASTEL_2;
			break;

		default:
			th->mode |= TRUEEMU_M_RGB;
			break;
	}
}


ggi_event_mask GGIeventpoll(ggi_visual *vis, ggi_event_mask mask,
			    struct timeval *tv)
{
	TrueemuHook *th = VIS_PRIVATE(vis);
	
	return ggiEventPoll(th->parent, mask, tv);
}


int GGIeventread(ggi_visual *vis, ggi_event *ev, ggi_event_mask mask)
{
	TrueemuHook *th = VIS_PRIVATE(vis);

	int err;

	for (;;) {
	
		err = ggiEventRead(th->parent, ev, mask);
		
		if (err < 0) {
			return err;
		}

		/* handle hotkeys */
		
		if (ev->any.type == evKeyPress)
		switch (ev->key.sym)
		{
			case U(K_NORMAL_CTRL):
				th->flags |= TRUEEMU_F_CTRL;
				break;
				
			case U(K_NORMAL_ALT):
				th->flags |= TRUEEMU_F_ALT;
				break;
			
			case U(K(KT_META, 'd' & 0x1f)):
				if ((th->flags & TRUEEMU_F_CTRL) &&
				    (th->flags & TRUEEMU_F_ALT)) {

					cycle_dither(th);
					trueemuNewMode(vis);
				}
				continue;  /* swallow it */
			
			case U(K(KT_META, 'p' & 0x1f)):
				if ((th->flags & TRUEEMU_F_CTRL) &&
				    (th->flags & TRUEEMU_F_ALT)) {

					cycle_palette(th);
					trueemuNewMode(vis);
				}
				continue;  /* swallow it */
		}

		if (ev->any.type == evKeyRelease)
		switch (ev->key.sym)
		{
			case U(K_NORMAL_CTRL):
				th->flags &= ~TRUEEMU_F_CTRL;
				break;
				
			case U(K_NORMAL_ALT):
				th->flags &= ~TRUEEMU_F_ALT;
				break;
		}

		return err;
	}
}
