/* 
   XFree86 Direct Graphics Access LibGGI target README:

   Programs need SUID root. You really should use KGI :-)

   This target makes a number of assumptions about the XFree86
   implementation of X Window System. So if DGA actually gets outside of
   XFree86 this target might not work. Blame it on XF86DGA's stupid design.

   When using the XF86DGA target, LibGGI programs will always segfault at
   the end. This happens because XF86DGA puts an atexit function to do some
   'cleanup' when the program exits.  However, the XF86DGA static library
   would be unloaded along with the target by ggiClose(), and calling the
   then-non-existent atexit function causes the segfault. A 'solution' used
   here is to _exit() when SIGSEGV is received.

   Likewise, XF86DGA will install a couple of new signal handlers; these are
   removed automatically on unloading of this target.

   See source for miscellaneous stuff.

   --

   Copyright (C) 1997 Steve Cheng <elmert@ipoline.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU Library General Public License as 
   published by the Free Software Foundation; either version 2, or 
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Library General Public License for more details.

   You should have received a copy of the GNU Library General Public 
   License along with this program; see the file COPYING.LGPL .  
   If not, write to the Free Software Foundation, 675 Mass Ave, 
   Cambridge, MA 02139, USA.
*/ 

#include <ggi/libggi.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>

#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/extensions/xf86dga.h>
#include <X11/extensions/xf86vmode.h>

/* Never prevent apps from segfaulting on exit */
/*
#define ALWAYS_SEGFAULT_ON_EXIT
*/

/* Macros to gain and give up root privs, for mmapping /dev/mem. Note that
   they use the non-POSIX (but BSD 4.3 conformant) call seteuid() because
   regular setuid() does not allow regaining root privs when they have
   already been given up. */

#define GET_ROOT	seteuid(0)
#define GIVE_UP_ROOT	seteuid(getuid())

/* VIS_PRIVATE info */

struct Xhooks { 
	Display *display;
	int screen;
	XF86VidModeModeInfo **modes;
	int num_modes;
	unsigned height;
	unsigned width;
	unsigned depth;
	Colormap cmap;
	int XLibLock;	/* SpinLock would probably be more secure, but ... */
        int nocols;	/* Number of colors in the colormap */
	ggi_color *colormap;
};

#define XLIB_PRIV(vis) ((struct Xhooks *)VIS_PRIVATE(vis))
