/* 
   Mystique userspace acceleration library

   Copyright (C) 1997 Matthias Grimrath - m.grimrath@tu-bs.de

   Anyone is welcome to improve this lib and send diffs to me.
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <string.h>

#include "ggi-dl.h"
#include "mystique-8-15-16-32.h"

int GGIdrawbox(ggi_visual *vis, int x, int y, int w, int h)
{
	DPRINT("drawbox: fg=0x%.8x, x=%d, y=%d, width=%d, height=%d, "
	       "dwgctl=%.8x\n",VIS_GC_FGCOLOR(vis),x,y,w,h,
	       TRAP | __blkmode | SOLID | ARZERO | SGNZERO | SHFTZERO |
	       BOP_COPY | BLTMOD_BMONOLEF);

	ACC_ADDDATA(DMA_YDSTLEN, (y << 16) | h);
	ACC_ADDDATA(DMA_FXBNDRY, ((x+w) << 16) | x);
	ACC_ADDDATA(DMA_FCOL, VIS_GC_FGCOLOR(vis));
	ACC_ADDDATA(DMA_DWGCTL | DMA_GO, TRAP | __blkmode | SOLID | ARZERO | 
		    SGNZERO | SHFTZERO | BOP_COPY | BLTMOD_BMONOLEF);
	
	return SYNC_FLUSH(vis);
}

/*
** This small ASCII chart is supposed to help you understand that
** wild "if" block. The origin is (x,y), i.e. the top left corner of
** the source.
** 
**           top-down
**              |
** left-right---+---right-left
**              |
**          bottom-up
*/
int GGIcopybox(ggi_visual *vis, int x, int y, int w, int h, int nx, int ny)
{
	ggi_uint lefttop;

	DPRINT("copybox: x=%d, y=%d, w=%d, h=%d nx=%d ny=%d\n",x,y,w,h,nx,ny);
	
	/* Checking values is for cowards ... ;) */
	if (ny < y) { /* top-down */

		ACC_ADDDATA(DMA_AR5, VIS_DB(vis)->buffer.pixellinearbuffer.size.x);
		lefttop = (y * VIS_DB(vis)->
			   buffer.pixellinearbuffer.size.x) + x; 
		if (nx < x) { /* left-right */
			ACC_ADDDATA(DMA_AR3, lefttop);
			ACC_ADDDATA(DMA_AR0, lefttop + w - 1);
			ACC_ADDDATA(DMA_SGN, 0);
		} else { /* right-left */
			ACC_ADDDATA(DMA_AR3, lefttop + w - 1);
			ACC_ADDDATA(DMA_AR0, lefttop);
			ACC_ADDDATA(DMA_SGN, SGN_SCANLEFT);
       		}
		ACC_ADDDATA(DMA_YDSTLEN, (ny << 16) | h);
		
	} else { /* bottom-up */
		
		ACC_ADDDATA(DMA_AR5, -(VIS_DB(vis)->buffer.pixellinearbuffer.size.x));
		lefttop = ((y+h-1) * VIS_DB(vis)->
			   buffer.pixellinearbuffer.size.x) + x; 
		if (nx < x) { /* left-right */
			ACC_ADDDATA(DMA_AR3, lefttop);
			ACC_ADDDATA(DMA_AR0, lefttop + w - 1);
			ACC_ADDDATA(DMA_SGN, SGN_SDY);
		} else { /* right-left */
			ACC_ADDDATA(DMA_AR3, lefttop + w - 1);
			ACC_ADDDATA(DMA_AR0, lefttop);
			ACC_ADDDATA(DMA_SGN, SGN_SDY | SGN_SCANLEFT);
		}
		ACC_ADDDATA(DMA_YDSTLEN, ((ny+h-1) << 16) | h);
	}
	ACC_ADDDATA(DMA_FXBNDRY, ((nx + w - 1) << 16) | nx);
	ACC_ADDDATA(DMA_DWGCTL | DMA_GO, BITBLT | ATYPE_RPL | SHFTZERO |
		    BOP_COPY | BLTMOD_BFCOL);
	  
	return SYNC_FLUSH(vis);
}
