/* 
 *  2D graphics library for GGI
 *
 *  Copyright (C) 1998 by Thomas Tanner. See CREDITS for details.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __GGI_GGI2D_H
#define __GGI_GGI2D_H

#include <ggi/libggi.h>

/*----------------------------------------------------------------------*/

typedef enum { 
	GGI_ARC_SECTOR,
	GGI_ARC_SEGMENT
} ggi_arcmode;

typedef enum { 
	GGI_POLY_EVENODD, 
	GGI_POLY_WINDING 
} ggi_polymode;

typedef enum {
	GGI_NOOP,			/* dest = dest */
	GGI_INVERT,			/* dest = ~dest */
	GGI_SET,			/* dest = color */
	GGI_SET_INVERTED,		/* dest = ~color */
	GGI_AND,			/* dest = (dest & color) */
	GGI_NAND,			/* dest = ~(dest & color) */
	GGI_AND_REVERSE,		/* dest = ~dest & color */
	GGI_AND_INVERTED,		/* dest = dest & ~color */
	GGI_OR,				/* dest = (dest | color) */
	GGI_NOR,			/* dest = ~(dest | color) */
	GGI_OR_REVERSE,			/* dest = ~dest & color */
	GGI_OR_INVERTED,		/* dest = dest & ~color */
	GGI_XOR,			/* dest = (dest ^ color) */
	GGI_EQUIV,			/* dest = ~(dest ^ color */
	GGI_ADD,			/* dest = dest + color */
	GGI_SUB				/* dest = dest - color */
} ggi_operator;

typedef	struct {
	int	x1, x2, y;
} ggi_scanline;

/*----------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C" {
#endif

int	ggi2dInit(void);
void	ggi2dExit(void);

int	ggi2dOpen(ggi_visual_t vis);
void	ggi2dClose(ggi_visual_t vis);
void	ggi2dReset(ggi_visual_t vis);

void	ggiSetClip(ggi_visual_t vis, int x1, int y1, int x2, int y2);
ggi_bool ggiPointVisible(ggi_visual_t vis, int x, int y);
ggi_bool ggiRectVisible(ggi_visual_t vis, int x1, int y1, int x2, int y2);

void	ggiSetArcMode(ggi_visual_t vis, ggi_arcmode mode);
ggi_arcmode ggiGetArcMode(ggi_visual_t vis);
void	ggiSetPolyMode(ggi_visual_t vis, ggi_polymode mode);
ggi_polymode ggiGetPolyMode(ggi_visual_t vis);

void	ggiSetLineDash(ggi_visual_t vis, uint dash[], uint size);
void	ggiGetLineDash(ggi_visual_t vis, uint dash[], uint *size);
void	ggiSetAppendMode(ggi_visual_t vis, ggi_bool append);
ggi_bool ggiGetAppendMode(ggi_visual_t vis);
void	ggiSetAntialias(ggi_visual_t vis, ggi_bool antialias);
ggi_bool ggiGetAntialias(ggi_visual_t vis);

void	ggiSetDrawColor(ggi_visual_t vis, ggi_pixel color);
ggi_pixel ggiGetDrawColor(ggi_visual_t vis);
void	ggiSetFillColor(ggi_visual_t vis, ggi_pixel color);
ggi_pixel ggiGetFillColor(ggi_visual_t vis);
void	ggiSetFillTexture(ggi_visual_t vis, int refx, int refy, ggi_visual_t texture);
ggi_visual_t ggiGetFillTexture(ggi_visual_t vis, int *refx, int *refy);

void	ggiSetOperator(ggi_visual_t vis, ggi_operator operator);
ggi_operator ggiGetOperator(ggi_visual_t vis);
void	ggiSetAlpha(ggi_visual_t vis, ggi_alpha alpha);
ggi_alpha ggiGetAlpha(ggi_visual_t vis);

/* primitives */

void	ggi2dPutPixel(ggi_visual_t vis, int x, int y, ggi_pixel color);
void	ggi2dDrawPixel(ggi_visual_t vis, int x, int y);
/* override libGGI routines */
#define	ggiPutPixel	ggi2dPutPixel
#define	ggiDrawPixel	ggi2dDrawPixel
void	ggiDrawPixels(ggi_visual_t vis, ggi_coord coords[], uint count);
void	ggiScanLine(ggi_visual_t vis, int x1, int x2, int y);
void	ggiScanLines(ggi_visual_t vis, ggi_scanline scanlines[], uint count);
void	ggiHLine(ggi_visual_t vis, int x1, int x2, int y);
void	ggiVLine(ggi_visual_t vis, int x, int y1, int y2);
void	ggiDrawRect(ggi_visual_t vis, int x1, int y1, int x2, int y2);
void	ggiFillRect(ggi_visual_t vis, int x1, int y1, int x2, int y2);

/* curves */

void	ggiLine(ggi_visual_t vis, int x1, int y1, int x2, int y2);
void	ggiLinef(ggi_visual_t vis, 
	    ggi_float x1, ggi_float y1, ggi_float x2, ggi_float y2);
void	ggiDrawLines(ggi_visual_t vis, ggi_line lines[], uint count);
void	ggiDrawCircle(ggi_visual_t vis, int x, int y, uint r);
void	ggiFillCircle(ggi_visual_t vis, int x, int y, uint r);
void	ggiDrawEllipse(ggi_visual_t vis, int x, int y, uint rx, uint ry);
void	ggiFillEllipse(ggi_visual_t vis, int x, int y, uint rx, uint ry);
void	ggiDrawArc(ggi_visual_t vis, int x, int y, uint rx, uint ry,
	    	   ggi_float startangle, ggi_float endangle, ggi_bool close);
void	ggiFillArc(ggi_visual_t vis, int x, int y, uint rx, uint ry,
		   ggi_float startangle, ggi_float endangle);
void	ggiBezier(ggi_visual_t vis, 
		    int x1, int y1, int x2, int y2,
		    int x3, int y3, int x4, int y4);

/* polygon drawing */

void	ggiTrapezoid(ggi_visual_t vis, 
		    int xl1, int xr1, int y1,
		    int xl2, int xr2, int y2);
void	ggiTriangle(ggi_visual_t vis, int x1, int y1, int x2, int y2, int x3, int y3);
void	ggiDrawPoly(ggi_visual_t vis, ggi_coord coords[], uint count);
void	ggiFillPoly(ggi_visual_t vis, ggi_coord coords[], uint count);
void	ggiFillPolys(ggi_visual_t vis, ggi_coord coords[], uint counts[], uint count);

/* blitting */

void	ggiBlit(
		    ggi_visual_t dest, int dx, int dy,
		    ggi_visual_t src, int sx, int sy, 
		    int width, int height);
void	ggiBlitTrans(
		    ggi_visual_t dest, int dx, int dy, 
		    ggi_visual_t src, int sx, int sy, 
		    int width, int height, ggi_pixel transcol);
void	ggiBlitOp(
		    ggi_visual_t dest, int dx, int dy, 
		    ggi_visual_t src1, ggi_visual_t src2, int sx, int sy, 
		    int width, int height, ggi_operator op);
void	ggiStretchBlit(		    
		    ggi_visual_t dest, int dx, int dy, int dwidth, int dheight, 
		    ggi_visual_t src, int sx, int sy, int swidth, int sheight);
void	ggiStretchBlitTrans(		    
		    ggi_visual_t dest, int dx, int dy, int dwidth, int dheight,
		    ggi_visual_t src, int sx, int sy, int swidth, int sheight,
		    ggi_pixel transcol);
void	ggiStretchBlitOp(		    
		    ggi_visual_t dest, int dx, int dy, int dwidth, int dheight,
		    ggi_visual_t src1, ggi_visual_t src2,
		    int sx, int sy, int swidth, int sheight,
		    ggi_operator op);

#ifdef __cplusplus
}
#endif

/*----------------------------------------------------------------------*/

#endif /* __GGI_GGI2D_H */
