/* 
 *  Line template
 *
 *  Copyright (C) 1998 by Thomas Tanner. See CREDITS for details.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*----------------------------------------------------------------------
macros that must be defined:
    
#define HLINE(x1, x2, y)
	draws a horizontal line from (x1,y) to (x2,y) 
#define VLINE(x, y1, y2)
	draws a vertical line from (x,y1) to (x,y2) 

#define PLOT(x, y)
	plots a pixel at (x,y)
#define PLOTA(x, y, alpha)
	plots a pixel at (x,y)
    
options that can be defined:
	
#define FLOAT_COORDS
	line with ggi_float coordinates
#define ANTIALIAS
	antialiased line
#define STIPPLE
	stippled line

----------------------------------------------------------------------*/

/* 
void	line(ggi_visual_t vis, int x1, int y1, int x2, int y2)
void	linef(ggi_visual_t vis, ggi_float x1, ggi_float y1, ggi_float x2, ggi_float y2)
*/
{
	uint	x, y;
	int	xstep;
	int	end;
#if defined(FLOAT_COORDS) || defined(ANTIALIAS)
	fpoint	fx, fy, d;
#else 
	int	d;
#endif
#ifdef FLOAT_COORDS
	fpoint	dx, dy, tmp;
#else
	int	dx, dy, tmp;
#endif
#ifdef STIPPLE
	uint	stipple_count;
#endif
#ifdef ANTIALIAS
	uint	alpha;
#endif
/*----------------------------------------------------------------------*/
/* initialize */

	if (y1 > y2) {
		xchg(x1, x2, tmp);
		xchg(y1, y2, tmp);
	}
	dx = x2 - x1;
	dy = y2 - y1;
	if (dx < 0) { dx = -dx; xstep = -1; } else xstep = 1;
#ifdef FLOAT_COORDS
	x = FPRound(x1); y = FPRound(y1);
#else
	x = x1; y = y1;
#endif
/*----------------------------------------------------------------------*/
/* horizontal line */

	if (dy == 0) 
#ifdef FLOAT_COORDS
		HLINE(x, FPRound(x2), y);
#else
		HLINE(x, x2, y);
#endif
/*----------------------------------------------------------------------*/
/* vertical line */

	else if (dx == 0)
#ifdef FLOAT_COORDS
		VLINE(x, y, FPRound(y2));
#else
		VLINE(x, y, y2);
#endif
/*----------------------------------------------------------------------*/
	else {
		PLOT(x, y);
/*----------------------------------------------------------------------*/
/* step in vertical direction */   

	if (dx < dy) {
#ifdef ANTIALIAS
#	ifdef	FLOAT_COORDS
			d = FPDiv(dx * xstep, dy);
			fx = x1; end = FPRound(y2);
#	else
			d = FPDivInt(dx * xstep, dy);
			fx = FPNewInt(x1); end = y2;
#	endif
		if (xstep == 1) 
			while (y != end) {
			fx += d; y++; x = FPFloor(fx);
				alpha = FPInt(FPFrac(fx) * GGI_MAX_ALPHA);
				PLOTA(x, y, GGI_MAX_ALPHA - alpha);
				PLOTA(x + 1, y, alpha);
			}
		else 
			while (y != end) {
			fx += d; y++; x = FPCeil(fx);
				alpha = FPInt(FPFrac(-fx) * GGI_MAX_ALPHA);
				PLOTA(x, y, GGI_MAX_ALPHA - alpha);
				PLOTA(x - 1, y, alpha);
			}
#else
#	ifdef	FLOAT_COORDS
		d = FPDiv(dx * xstep, dy);
		fx = x1; end = FPRound(y2);
		while (y != end) {
			y++; fx += d;
			PLOT(FPRound(fx), y);
		}
#	else
		d = dy - 1; dx += dx; dy += dy;
		while (y != y2) {
			y++; d -= dx;
			if (d < 0) { d += dy; x += xstep; }
			PLOT(x, y);
		}
#	endif
#endif
	} else 
/*----------------------------------------------------------------------*/
/* step in horizontal direction */   

	if (dx > dy) {	
#ifdef ANTIALIAS
#	ifdef	FLOAT_COORDS
		d = FPDiv(dy, dx);
		fy = y1; end = FPRound(x2);
#	else
		d = FPDivInt(dy, dx);
		fy = FPNewInt(y1); end = x2;
#	endif
		while (x != end) {
			fy += d; x += xstep; y = FPFloor(fy);
			alpha = FPInt(FPFrac(fy) * GGI_MAX_ALPHA);
			PLOTA(x, y, GGI_MAX_ALPHA - alpha);
			PLOTA(x, y + 1, alpha);
		}
#else
#	ifdef	FLOAT_COORDS
		d = FPDiv(dy, dx);
		fy = y1; end = FPRound(x2);
		while (x != end) {
			x += xstep; fy += d; 
			PLOT(x, FPRound(fy));
		}
#	else
		d = dx - 1; dx += dx; dy += dy; end = x2;
		while (x != end) {
			x += xstep; d -= dy;
			if (d < 0) { d += dx; y++; }
			PLOT(x, y);
		}
#	endif
#endif
	} else 
/*----------------------------------------------------------------------*/
/* diagonal line */
	{
#ifdef FLOAT_COORDS
		end = FPRound(y2);
#else
		end = y2;
#endif
		while (y != end) {
			x += xstep; y++;
			PLOT(x, y);
		}
	}
/*----------------------------------------------------------------------*/
	}
}

#undef HLINE
#undef VLINE
#undef PLOT
#undef PLOTA
#undef FLOAT_COORDS
#undef ANTIALIAS
#undef STIPPLE
