/* for compatiblity with old libggi */

#ifndef _structs_H_
#define _structs_H_

#include <sys/time.h>
#include <ggi/ggi.h>

#define _BUILDING_LIBGGI
#include <ggi/libggi.h>
#include <ggi/threads.h>

#define GGI_VERSION_DLL	2

/* Create a OP version number from the revision
 * (ABI compatability) and number of functions
 */
#define	GGI_VERSION_MK(num_funcs)	  ((GGI_VERSION_DLL<<16)|num_funcs)
#define GGI_VERSION_REV(vers)		  (((unsigned int)vers)>>16)
#define GGI_VERSION_FUNCS(vers)	  ((vers) & 0xFFFF)

/* Increment the version number every time you add 
 * new features.
 */
#define GGI_GC_FLAG_DIRTY	0x0001	

struct ggi_visual;

typedef struct ggi_dlhandle {
	void   *handle;
	char   *name;
	int	use;
	int 	type;
	struct ggi_visual *visual;
	int	(*init)(struct ggi_visual *vis,const char *args);
	int	(*cleanup)(struct ggi_visual *vis);
	ggi_lib_func (*privfunc)(struct ggi_visual *vis,ggi_uint funcid);
} ggi_dlhandle;

typedef struct ggi_dlhandle_l {
	struct ggi_dlhandle *handle;
	struct ggi_dlhandle_l *next;
} ggi_dlhandle_l;

typedef struct ggi_extension {
	int size;
	int (*load)(ggi_visual_t,const char *,const char *);
	int (*release)(ggi_visual_t, const char *);
	struct ggi_extension *next,*prev;
} ggi_extension;

#define GGI_VERSION_VISUAL	GGI_VERSION_MK(20)
typedef struct ggi_visual {
	ggi_uint version;		
	ggi_mutex	mutex;	/* Lock when changing.. */
	struct ggi_visual *next; /* Next one in the list... */

	/* Some misc data items [5/10] */
	int fd;			/* File descriptor of device/fifo/file */
	ggi_gc *gc;		/* Graphics context */
	void  *evqueue;		/* Event queue */
	void  *priv;		/* Display private data */
	ggi_dlhandle_l	*dlhandle;	/* Handles to dynamic libs */
	void  *dummy[5];	/* For future expansion */

	/* Operation structs [4/10] */
	struct ggi_visual_opdisplay *opdisplay;	/* Display open/mode setting */
	struct ggi_visual_opcolor   *opcolor;	/* Color & Palette Ops */
	struct ggi_visual_op2d      *op2d;	/* 2D operations */
	void  			    *opdummy[6];/* For future expansion */

	struct ggi_info info;		/* Info on the mode */
	int size; 			/* Size of all extensions */
	ggi_dlhandle_l	*extlib;	/* Dynamic libs from extensions */

} ggi_visual;

typedef struct ggi_focus {
	ggi_mutex	mutex;	/* Lock when changing.. */
	ggi_visual *focus;	/* Visual that this struct represents */
	int visuals;		/* # of visuals active */
	ggi_visual *visual;	/* Linked list of all visuals */
} ggi_focus;

#define GGI_DL_ERROR		0x80000000
#define GGI_DL_OPDISPLAY	0x00000001
#define GGI_DL_OPCOLOR		0x00000002
#define GGI_DL_OP2D		0x00000004
#define GGI_DL_EXTENSION	0x00000008

#define GGI_DLTYPE_INTERNAL 	0x01
#define GGI_DLTYPE_EXTENSION	0x02

	/* int  GGIdlinit(ggi_visual *vis,const char *arg) 
	 *  Returns the mask of the `vis->op*' types it modified,
	 * or GGI_DL_ERROR|errorcode if error 
	 */
#define GGI_DLINIT_SYM	"GGIdlinit"	
	/* void GGIdlcleanup(ggi_visual *vis) */
#define GGI_DLCLEANUP_SYM "GGIdlcleanup"	
	/* int (*)(...) GGIdlprivfunc(ggi_visual *vis,ggi_uint funcid) */
#define GGI_DLPRIVFUNC_SYM "GGIdlprivfunc"

/* Display and mode setting management
 * Increment the function number every time you add a function.
 */
#define GGI_VERSION_VISUAL_OPDISPLAY GGI_VERSION_MK(15)
struct ggi_visual_opdisplay {
	ggi_uint		version;
	ggi_dlhandle_l *dlhandle;
	int 		(*null)(void);	/* Placeholder... */

/*------- Section: Misc management (2/5 slots) -------------*/
	int 		(*dummy)(void);	/* Place holder */

	/* Flush all pending accel events */
	int (*flush)(ggi_visual *vis);

	int (*dummy_focus[3])(void);	/* Place holder */

/*------- Section: Mode management (3/5 slots) -------------*/
	/* Get the display mode */
	int (*getmode)(ggi_visual *vis,ggi_mode *tm);
	/* Check the display mode */
	int (*checkmode)(ggi_visual *vis,ggi_mode *tm);
	/* Set the display mode */
	int (*setmode)(ggi_visual *vis,ggi_mode *tm);

	int (*dummy_mode[2])(void);	/* Place holder */

/*------- Section: Event management (2/5 slots) ------------*/
	/* Poll for events */
	ggi_event_mask (*eventpoll)(ggi_visual *vis,
				    ggi_event_mask mask, 
				    struct timeval *timeout);
	/* Read a single event */
	int (*eventread)(ggi_visual *vis, ggi_event *ev, ggi_event_mask mask);

	int (*dummy_event[3])(void);

};

/* Color and palette management
 * Increment the function number every time you add a function.
 */
#define GGI_VERSION_VISUAL_OPCOLOR GGI_VERSION_MK(20)
struct ggi_visual_opcolor {
	ggi_uint		 version;
	ggi_dlhandle_l *dlhandle;
	int 		(*null)(void);	/* Placeholder... */

/*------- Section: Color (8/10 slots) -------------*/
	/* Map a color into a ggi_uint - used for the drawinfo structure */	
	ggi_pixel (*mapcolor)(ggi_visual *vis,ggi_color *color);

	/* Map a ggi_uint into a color */	
	int (*unmappixel)(ggi_visual *vis,ggi_pixel pixel,ggi_color *color);

	/* Pack a len size array of ggi_color into buffer - use getbpp! */
	int (*packcolors)(ggi_visual *vis,void *buffer,ggi_color *colors,int len);

	/* Unpack an array of pixels into an array of colors */
	int (*unpackpixels)(ggi_visual *vis,void *buffer,ggi_color *colors,int len);

	/* Set palette */
	int (*setpalvec)(ggi_visual *vis,int start,int len,ggi_color *colormap);

	/* Get palette */
	int (*getpalvec)(ggi_visual *vis,int start,int len,ggi_color *colormap);

	/* Convert a len size array of packed pixels to ggi_pixels.  */
	int (*screen2pixels)(ggi_visual *vis,void *input,ggi_pixel *output,int len);

	/* Convert a len size array of ggi_pixels to packed pixels.  */
	int (*pixels2screen)(ggi_visual *vis,ggi_pixel *input,void *output,int len);

	int (*dummy_color[2])(void);		/* Place holder */

/*------- Section: Gamma (4/10 slots) -------------*/
	/* Set the gamma value */
	int (*getgamma)(ggi_visual *vis,ggi_float *gamma_red,
				       ggi_float *gamma_green,
				       ggi_float *gamma_blue);

	/* Set the gamma value */
	int (*setgamma)(ggi_visual *vis,ggi_float gamma_red,
				       ggi_float gamma_green,
				       ggi_float gamma_blue);

	/* Get the gamma map explicitly */
	int (*getgammamap)(ggi_visual *vis,int start,int len,ggi_color *map);

	/* Set the gamma map explicitly */
	int (*setgammamap)(ggi_visual *vis,int start,int len,ggi_color *map);


	int (*dummy_gamma[6])(void);		/* Place holder */
};


/* 2D Drawing functions
 * Increment the function number every time you add a function.
 */
#define GGI_VERSION_VISUAL_OP2D	GGI_VERSION_MK(66)
struct ggi_visual_op2d {
	ggi_uint		 version;
	ggi_dlhandle_l *dlhandle;
	int 		(*null)(void);	/* Placeholder... */

/*------- Section: Framebuffer (4/10 slots) -------------*/
	/* Open fb */
	int (*openfb)(ggi_visual *vis);
	/* Close fb */
	int (*closefb)(ggi_visual *vis);

	int (*setorigin)(ggi_visual *vis,int x,int y);	
	int (*setsplitline)(ggi_visual *vis,int y);			/* Splitline */

	int (*dummy_fb[6])(void);	/* Place holders */

/*------- Section: Sprites (0/10 slots) -------------*/

	int (*dummy_spr[10])(void);	/* Place holders */

/*------- Section: Misc graphics (3/10 slots) -------------*/
	/* Generic graphics routines 
	 */

	int (*fillscreen)(ggi_visual *vis);

	int (*putc)(ggi_visual *vis,int x,int y,char c);
	int (*puts)(ggi_visual *vis,int x,int y,const char *string);

	int (*dummy_misc[7])(void);	/* Place holders */

/*------- Section: Pixel graphics (4/10 slots) -------------*/
	int (*drawpixel)(ggi_visual *vis,int x,int y);
	int (*putpixel)(ggi_visual *vis,int x,int y,ggi_pixel pixel);
	int (*getpixel)(ggi_visual *vis,int x,int y,ggi_pixel *pixel);

	int (*_drawpixel)(ggi_visual *vis,int x,int y);

	int (*dummy_pixel[6])(void);	/* Place holders */

/*------- Section: Simple Line graphics (9/15 slots) -------------*/
	int (*drawline)(ggi_visual *vis,int xs,int ys,int xe,int ye);

	int (*drawhline)(ggi_visual *vis,int x,int y,int width);
	int (*puthline)(ggi_visual *vis,int x,int y,int width,void *buffer);
	int (*gethline)(ggi_visual *vis,int x,int y,int width,void *buffer);

	int (*drawvline)(ggi_visual *vis,int x,int y,int width);
	int (*putvline)(ggi_visual *vis,int x,int y,int width,void *buffer);
	int (*getvline)(ggi_visual *vis,int x,int y,int width,void *buffer);

  	int (*_drawhline)(ggi_visual *vis,int x,int y,int width);
	int (*_drawvline)(ggi_visual *vis,int x,int y,int width);

	int (*dummy_line[6])(void);	/* Place holders */

/*------- Section: Box blitting (5/10 slots) -------------*/
	int (*drawbox)(ggi_visual *vis,int x,int w,int width,int height);
	int (*putbox)(ggi_visual *vis,int x,int y,int width,int height,void *buffer);
	int (*getbox)(ggi_visual *vis,int x,int y,int width,int height,void *buffer);
	int (*copybox)(ggi_visual *vis,int x,int y,int width,int height,int newx,int newy);
	int (*crossblit)(ggi_visual *src,int sx,int sy,int swidth,int sheight,
			 ggi_visual *dst,int dx,int dy,int dwidth,int dheight);

	int (*dummy_blit[5])(void);	/* Place holders */
};

#define	GGINAME(name)	GGI2D##GGILIBNAME##name
#define	GGILIB_INIT		GGIdlinit
#define	GGILIB_CLEANUP		GGIdlcleanup

#endif /* _structs_H_ */
