/* 
 *  API stubs
 *
 *  Copyright (C) 1998 Thomas Tanner. See CREDITS for details.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "internal.h"

/*----------------------------------------------------------------------*/

/* drawing functions */

#define	CALL(vis,f)	\
	_ggiLock(vis->mutex);\
	GGI2D_FUNC(vis)->f;\
	_ggiUnlock(vis->mutex);

void	ggi2dPutPixel(ggi_visual_t vis, int x, int y, ggi_pixel color)
{	CALL(vis,putPixel(vis, x, y, color))	}

void	ggi2dDrawPixel(ggi_visual_t vis, int x, int y)
{	CALL(vis,drawPixel(vis, x, y)) }

void	ggiDrawPixels(ggi_visual_t vis, ggi_coord coords[], uint count)
{	CALL(vis,drawPixels(vis, coords, count)) }

void	ggiHLine(ggi_visual_t vis, int x1, int x2, int y)
{	CALL(vis,hLine(vis, x1, x2, y)) }

void	ggiScanline(ggi_visual_t vis, int x1, int x2, int y)
{	CALL(vis,scanline(vis, x1, x2, y)) }

void	ggiScanlines(ggi_visual_t vis, ggi_scanline scanlines[], uint count)
{	CALL(vis,scanlines(vis, scanlines, count)) }

void	ggiVLine(ggi_visual_t vis, int x, int y1, int y2)
{	CALL(vis,vLine(vis, x, y1, y2)) }

void	ggiDrawRect(ggi_visual_t vis, int x1, int y1, int x2, int y2)
{	CALL(vis,drawRect(vis, x1, y1, x2, y2)) }

void	ggiFillRect(ggi_visual_t vis, int x1, int y1, int x2, int y2)
{	CALL(vis,fillRect(vis, x1, y1, x2, y2)) }

void	ggiLine(ggi_visual_t vis, int x1, int y1, int x2, int y2)
{	CALL(vis,line(vis, x1, y1, x2, y2)) }

void	ggiLinef(ggi_visual_t vis, ggi_float x1, ggi_float y1, ggi_float x2, ggi_float y2)
{	CALL(vis,linef(vis, x1, y1, x2, y2)) }

void	ggiDrawLines(ggi_visual_t vis, ggi_line lines[], uint count)
{	CALL(vis,drawLines(vis, lines, count)) }

void	ggiDrawCircle(ggi_visual_t vis, int x, int y, uint r)
{	CALL(vis,drawCircle(vis, x, y, r)) }

void	ggiFillCircle(ggi_visual_t vis, int x, int y, uint r)
{	CALL(vis,fillCircle(vis, x, y, r)) }

void	ggiDrawEllipse(ggi_visual_t vis, int x, int y, uint a, uint b)
{	CALL(vis,drawEllipse(vis, x, y, a, b)) }

void	ggiFillEllipse(ggi_visual_t vis, int x, int y, uint a, uint b)
{	CALL(vis,fillEllipse(vis, x, y, a, b)) }

void	ggiDrawArc(ggi_visual_t vis, int x, int y, uint a, uint b,
	    	   ggi_float startangle, ggi_float endangle, ggi_bool close)
{	CALL(vis,drawArc(vis, x, y, a, b, startangle, endangle, close)) }

void	ggiFillArc(ggi_visual_t vis, int x, int y, uint a, uint b,
		   ggi_float startangle, ggi_float endangle)
{	CALL(vis,fillArc(vis, x, y, a, b, startangle, endangle)) }

void	ggiBezier(ggi_visual_t vis, 
		    int x1, int y1, int x2, int y2,
		    int x3, int y3, int x4, int y4)
{	CALL(vis,bezier(vis, x1, y1, x2, y2, x3, y3, x4, y4)) }

void	ggiTrapezoid(ggi_visual_t vis, 
		    int xl1, int xr1, int y1,
		    int xl2, int xr2, int y2)
{	CALL(vis,trapezoid(vis, xl1, xr1, y1, xl2, xr2, y2)) }

void	ggiTriangle(ggi_visual_t vis, int x1, int y1, int x2, int y2, int x3, int y3)
{	CALL(vis,triangle(vis, x1, y1, x2, y2, x3, y3)) }

void	ggiDrawPoly(ggi_visual_t vis, ggi_coord coords[], uint count)
{	CALL(vis,drawPoly(vis, coords, count)) }

void	ggiFillPoly(ggi_visual_t vis, ggi_coord coords[], uint count)
{	CALL(vis,fillPoly(vis, coords, count)) }

void	ggiFillPolys(ggi_visual_t vis, ggi_coord coords[], uint counts[], uint count)
{	CALL(vis,fillPolys(vis, coords, counts, count)) }

void	ggiBlit(
		    ggi_visual_t dest, int dx, int dy,	
		    ggi_visual_t src, int sx, int sy, int width, int height)	
{	CALL(dest,blit(dest, dx, dy, src, sx, sy, width, height)) }

void	ggiBlitOp(
		    ggi_visual_t dest, int dx, int dy, 
		    ggi_visual_t src1, ggi_visual_t src2, 
		    int sx, int sy, int width, int height, 
		    ggi_operator op)
{	CALL(dest,blitOp(dest, dx, dy, src1, src2, sx, sy, width, height, op)) }

void	ggiStretchBlit(		    
		    ggi_visual_t dest, int dx, int dy, int dwidth, int dheight, 
		    ggi_visual_t src, int sx, int sy, int swidth, int sheight)
{	CALL(dest,stretchBlit(dest, dx, dy, dwidth, dheight, 
			src, sx, sy, swidth, sheight)) }

void	ggiStretchBlitOp(		    
		    ggi_visual_t dest, 
		    int dx, int dy, int dwidth, int dheight,
		    ggi_visual_t src1, ggi_visual_t src2,
		    int sx, int sy, int swidth, int sheight,
		    ggi_operator op)
{	CALL(dest,stretchBlitOp(dest, dx, dy, dwidth, dheight, 
			src1, src2, sx, sy, swidth, sheight, op)) }


