/* -----------------------------------------------------------------------------
**	GGI kernel font support
** -----------------------------------------------------------------------------
**
**	Copyright (C) 1995-1997 by Steffen Seeger [seeger@physik.tu-chemnitz.de]
**
**	This program is free software; you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation; either version 2, or (at your option)
**	any later version.
**
**	You should have received a copy of the GNU General Public License
**	along with this program; see the file COPYING.  If not, write to
**	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
** ---------------------------------------------------------------------------
*/

#define	MAINTAINER	"Steffen Seeger, [seeger@physik.tu-chemnitz.de]"

#include <ggi/kgi.h>

#include <linux/config.h>

#define	KGI_8x8_FONT
#define	KGI_8x14_FONT
#define	KGI_9x16_FONT

#include "default/font.inc"

static struct kgi_font *font[MAX_NR_DEFAULT_FONTS] =
{	/* !!! FIXME: for now there must be only one font per height*/
	#ifdef KGI_9x16_FONT

		&font_9x16,
	#endif

	#ifdef KGI_8x14_FONT

		&font_8x14,
	#endif

	#ifdef KGI_8x8_FONT

		&font_8x8,
	#endif
};

struct kgi_font *kgi_default_font(ggi_mode *mode)
{
	int i = 0;

	while (font[i] && (font[i]->dy > mode->dpp.y)) {

		i++;
	}

	return font[i];
}

ggi_uint kgi_fontpos(struct kgi_font *font, isochar sym)
{
	int row  = ISO_ROW(sym);
	int cell = ISO_CELL(sym);
	ggi_cellinfo *cellinfo = font->info->cellinfo[row];
	ggi_uint l = 0, h, m;

	if (row == 0xF0) {		/* display directly */

		return cell;
	}

	if ((sym & (ISO_GROUP_MASK | ISO_PLANE_MASK)) || !cellinfo) {

		return font->info->default_pos;
	}

	if (test_bit(row, font->info->map_direct)) {

		return ((uchar *)cellinfo) [cell];
	}

	if (sym > cellinfo->sym) {

		return font->info->default_pos;	/* symbol not in list?	*/
	}

	h = cellinfo->pos;
	cellinfo++;

	if (sym < cellinfo->sym) {

		return font->info->default_pos;
	}

	if (cellinfo->sym == sym) {		

		return cellinfo->pos;
	}

	do {
		/*	search matching cell
		*/
		if (cellinfo[h].sym == sym) {

			return cellinfo[h].pos;
		}

		m = (--h + l) >> 1;

		if (sym > cellinfo[m].sym) {

			l = m;

		} else { 

			h = m;
		}

	} while (l < h); 

	return font->info->default_pos;	/* no position defined		*/
}

isochar kgi_isochar(struct kgi_font *font, ggi_uint pos)
{
	return font->info->inversemap[(pos < font->info->positions) ? pos : 0];
}
